@@ -1,8 +1,7 @@
 //
-// $Id: AbsObjectCollector.cpp 4139 2007-06-22 14:13:04Z bakerj $
 //
 //****************************************************************************************//
-// Copyright (c) 2002-2007, The MITRE Corporation
+// Copyright (c) 2002-2008, The MITRE Corporation
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without modification, are
@@ -69,7 +68,7 @@ CollectedObject* AbsObjectCollector::Run(string objectId) {
 			collectedObject = CollectedObject::CreateError(objectId);
 			collectedObject->SetFlag(ex.GetVariable()->GetFlag());
 			
-			// add all the messages reported wtih the AbsVariable
+			// add all the messages reported with the AbsVariable
 			StringVector::iterator iterator;
 			for(iterator = ex.GetVariable()->GetMessages()->begin(); iterator != ex.GetVariable()->GetMessages()->end(); iterator++) {
 				collectedObject->AppendOvalMessage(new OvalMessage((*iterator)));
@@ -212,15 +211,17 @@ CollectedSet* AbsObjectCollector::Intersection(CollectedSet* collectedSet1, Coll
 	ItemVector* itemSet1 = collectedSet1->GetItems();
 	ItemVector* itemSet2 = collectedSet2->GetItems();
 
+	// Add the items from set 1 that exist in set 2
 	ItemVector::iterator iterator;
 	for(iterator = itemSet1->begin(); iterator != itemSet1->end(); iterator++) {
-		if(!this->ExistsInSet(itemSet2, (*iterator))) {
+		if(this->ExistsInSet(itemSet2, (*iterator))) {
 			resultItems->push_back((*iterator));
 		}
 	}
 
+	// Add the items from set 2 that exist in set 1
 	for(iterator = itemSet2->begin(); iterator != itemSet2->end(); iterator++) {
-		if(!this->ExistsInSet(itemSet1, (*iterator))) {
+		if(this->ExistsInSet(itemSet1, (*iterator))) {
 			resultItems->push_back((*iterator));
 		}
 	}
@@ -265,17 +266,7 @@ CollectedObject* AbsObjectCollector::Process(AbsObject* absObject) {
 }
 
 CollectedObject* AbsObjectCollector::ProcessSetObject(SetObject* setObject) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	 - Copy the variable values from the specified object.
-	//	 - Make call to process set to start the recursive processing of the set.
-	//	 - Set the references based on the results of processing the set 
-	//	 - Set the flag based on the results of processing the set
-	//	 - Copy the variable values used to create the collected set
-	// -----------------------------------------------------------------------
 
-	
 	CollectedSet* collectedSet = this->ProcessSet(setObject->GetSet());
 	CollectedObject* collectedObject = CollectedObject::Create(setObject);	
 	collectedObject->SetReferences(collectedSet->GetItems());
@@ -286,13 +277,6 @@ CollectedObject* AbsObjectCollector::ProcessSetObject(SetObject* setObject) {
 }
 
 CollectedSet* AbsObjectCollector::ProcessSet(Set* set) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//	
-	//	Either recursivley process each child set or Process each object 
-	//	reference.
-	//	
-	// -----------------------------------------------------------------------
 
 	CollectedSet* collectedSet1 = NULL;
 	CollectedSet* collectedSet2 = NULL;
@@ -310,11 +294,9 @@ CollectedSet* AbsObjectCollector::ProcessSet(Set* set) {
 
 	} else {
 
-		collectedSet1 = new CollectedSet();
-		collectedSet2 = new CollectedSet();
-
 		// Collect each referenced object using any supplied filters
 		if(set->GetReferenceOne() != NULL) {
+			collectedSet1 = new CollectedSet();
 			CollectedObject* refOneCollectedObj = this->Run(set->GetReferenceOne()->GetId());
 			ItemVector* itemSet1 = refOneCollectedObj->GetReferences();
 			itemSet1 = this->ApplyFilters(itemSet1, set->GetFilters());
@@ -325,6 +307,7 @@ CollectedSet* AbsObjectCollector::ProcessSet(Set* set) {
 		}
 
 		if(set->GetReferenceTwo() != NULL) {
+			collectedSet2 = new CollectedSet();
 			CollectedObject* refTwoCollectedObj = this->Run(set->GetReferenceTwo()->GetId());
 			ItemVector* itemSet2 = refTwoCollectedObj->GetReferences();
 			itemSet2 = this->ApplyFilters(itemSet2, set->GetFilters());
@@ -337,7 +320,7 @@ CollectedSet* AbsObjectCollector::ProcessSet(Set* set) {
 
 	// combine all sets by operator
 	CollectedSet* combinedCollectedSet = NULL;
-	if(set->GetReferenceTwo() != NULL) {
+	if(collectedSet2 != NULL) {
 		if(set->GetSetOperator() == OvalEnum::SET_OPERATOR_UNION) {
 			combinedCollectedSet = this->Union(collectedSet1, collectedSet2);
 		} else if(set->GetSetOperator() == OvalEnum::SET_OPERATOR_COMPLEMENT) {
@@ -362,16 +345,6 @@ CollectedSet* AbsObjectCollector::ProcessSet(Set* set) {
 }
 
 CollectedObject* AbsObjectCollector::ProcessObject(Object* object) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	 - make sure the object is supported and applicable
-	//	 - copy any variable values from the object
-	//	 - Get the probe for the object
-	//	 - Run the probe with the object
-	//	 - Set the matching collected items for the collected object
-	//	 - Set the flag value for the collected object.
-	// -----------------------------------------------------------------------
 
 	CollectedObject* collectedObject = NULL;
 
@@ -395,30 +368,12 @@ CollectedObject* AbsObjectCollector::ProcessObject(Object* object) {
 				// only create collected object if the pribe succeeds
 				collectedObject = CollectedObject::Create(object);
 				collectedObject->AppendVariableValues(object->GetVariableValues());
-				collectedObject->AppendReferences(items);
+				collectedObject->AppendReferencesAndComputeFlag(items);
 			} else {
+				
+				// because we first check if the object is supported the code should never get here.
 				throw AbsObjectCollectorException("Error: Unable to locate a probe for the specified object.", ERROR_FATAL);
 			}
-
-			// set the flag corerctly
-			if(collectedObject->GetReferences()->size() == 0) {
-				collectedObject->SetFlag(OvalEnum::FLAG_DOES_NOT_EXIST);
-			} else {
-
-				//
-				//
-				// TODO: here is where i need to figure out a new way to set the flag.
-				// If there are collected objects the flag may still be does not exit.
-				// need to check that one more more items has a status of exists
-				//
-				// Then i need to review each probe. When ever a probe detects that 
-				// an item is not found it needs to report it rather than reporting 
-				// nothing
-				//
-				//
-
-				collectedObject->SetFlag(OvalEnum::FLAG_COMPLETE);
-			}
 		}
 	}
 
@@ -426,11 +381,6 @@ CollectedObject* AbsObjectCollector::ProcessObject(Object* object) {
 }
 
 OvalEnum::Flag AbsObjectCollector::CombineFlagBySetOperator(OvalEnum::SetOperator setOp, OvalEnum::Flag set1Flag, OvalEnum::Flag set2Flag) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Combine the 
-	// -----------------------------------------------------------------------
 
 	OvalEnum::Flag result = OvalEnum::FLAG_ERROR;
 
@@ -622,23 +572,9 @@ OvalEnum::Flag AbsObjectCollector::CombineFlagBySetOperator(OvalEnum::SetOperato
 //						AbsObjectCollectorException Class								  //	
 //****************************************************************************************//
 AbsObjectCollectorException::AbsObjectCollectorException(string errMsgIn, int severity, Exception* ex) : Exception(errMsgIn, severity, ex) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Set the error message and then set the severity to ERROR_FATAL. This is 
-	//	done with the explicit call to the Exception class constructor that 
-	//	takes a single string param.
-	//
-	// -----------------------------------------------------------------------
 
 }
 
 AbsObjectCollectorException::~AbsObjectCollectorException() {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Do nothing for now
-	//
-	// -----------------------------------------------------------------------
 
 }
