@@ -1,8 +1,7 @@
 //
-// $Id: Analyzer.cpp 4121 2007-06-22 13:55:59Z bakerj $
 //
 //****************************************************************************************//
-// Copyright (c) 2002-2007, The MITRE Corporation
+// Copyright (c) 2002-2008, The MITRE Corporation
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without modification, are
@@ -33,6 +32,9 @@
 
 DOMElement* Analyzer::definitionsElm = NULL;
 DOMElement* Analyzer::testsElm = NULL;
+DOMElement* Analyzer::resultsSystemElm = NULL;
+DOMElement* Analyzer::resultsElm = NULL;
+
 StringPairVector Analyzer::trueResults;
 StringPairVector Analyzer::falseResults;
 StringPairVector Analyzer::unknownResults;
@@ -45,8 +47,7 @@ StringPairVector Analyzer::notApplicableResults;
 //****************************************************************************************//
 
 Analyzer::Analyzer() {
-	this->resultsSystemElm = NULL;
-	this->resultsElm = NULL;
+
 }
 
 Analyzer::~Analyzer() {
@@ -55,14 +56,38 @@ Analyzer::~Analyzer() {
 // ***************************************************************************************	//
 //								Public members												//
 // ***************************************************************************************	//
+DOMElement* Analyzer::GetResultsElm() {	
+	return Analyzer::resultsElm;
+}
 
-DOMElement* Analyzer::GetResultsTestsElm() {
+DOMElement* Analyzer::GetResultsSystemElm() {	
+
+	if(Analyzer::resultsSystemElm == NULL) {
+		DOMElement *elm = XmlCommon::AddChildElement(DocumentManager::GetResultDocument(), Analyzer::GetResultsElm(), "system");
+		Analyzer::resultsSystemElm = elm;
+	}
+	return Analyzer::resultsSystemElm;
+}
+
+DOMElement* Analyzer::GetResultsSystemDefinitionsElm() {	
+
+	if(Analyzer::definitionsElm == NULL) {
+		DOMElement *elm = XmlCommon::AddChildElement(DocumentManager::GetResultDocument(), Analyzer::GetResultsSystemElm(), "definitions");
+		Analyzer::definitionsElm = elm;
+	}
+	return Analyzer::definitionsElm;
+}
+
+DOMElement* Analyzer::GetResultsSystemTestsElm() {
+	
+	if(Analyzer::testsElm == NULL) {
+		DOMElement *elm = XmlCommon::AddChildElement(DocumentManager::GetResultDocument(), Analyzer::GetResultsSystemElm(), "tests");
+		Analyzer::testsElm = elm;
+	}
 	return Analyzer::testsElm;
 }
 
-DOMElement* Analyzer::GetResultsDefinitionsElm() {
-	return Analyzer::definitionsElm;
-}
+
 
 void Analyzer::AppendTrueResult(StringPair* pair) {
 	return Analyzer::trueResults.push_back(pair);
@@ -92,7 +117,7 @@ void Analyzer::Run() {
 
 	this->InitResultsDocument();
 
-	// get the definitions element in the defintions file
+	// get the definitions element in the definitions file
 	int prevIdLength = 1;
 	int curIdLength = 1;
 	DOMElement* definitionsElm = XmlCommon::FindElementNS(DocumentManager::GetDefinitionDocument(), "definitions");
@@ -127,11 +152,9 @@ void Analyzer::Run() {
 						cout << backSpaces << definitionId << blankSpaces << endBackSpaces;
 					}
 
-					Definition* def = new Definition();
-					def->Parse(definitionElm);
-					//OvalEnum::ResultEnumeration result = def->Analyze();
+					Definition* def = Definition::GetDefinitionById(definitionId);
 					def->Analyze();
-					def->Write(Analyzer::GetResultsDefinitionsElm());					
+					def->Write(Analyzer::GetResultsSystemDefinitionsElm());					
 					prevIdLength = definitionId.length();
 				}
    			}
@@ -156,11 +179,11 @@ void Analyzer::Run() {
 		VariableValue::ClearCache();
 
 		this->FinializeResultsDocument();
+
 	} else {
-		string logMessage = "\n    No definitions found! \n";
+		string logMessage = "\n    No definitions found in the input oval-definitions document! \n";
 		cout << logMessage;
 		Log::UnalteredMessage(logMessage);
-		throw AnalyzerException("");
 	}
 }
 
@@ -203,10 +226,9 @@ void Analyzer::Run(StringVector* definitionIds) {
 						cout << backSpaces << definitionId << blankSpaces << endBackSpaces;
 					}
 
-					Definition* def = new Definition();
-					def->Parse(definitionElm);
+					Definition* def = Definition::GetDefinitionById(definitionId);
 					def->Analyze();
-					def->Write(Analyzer::GetResultsDefinitionsElm());					
+					def->Write(Analyzer::GetResultsSystemDefinitionsElm());					
 					prevIdLength = definitionId.length();
 
 				} else {
@@ -243,10 +265,9 @@ void Analyzer::Run(StringVector* definitionIds) {
 						cout << backSpaces << definitionId << blankSpaces << endBackSpaces;
 					}
 
-					Definition* def = new Definition();
-					def->Parse(definitionElm);
+					Definition* def = Definition::GetDefinitionById(definitionId);
 					def->NotEvaluated();
-					def->Write(Analyzer::GetResultsDefinitionsElm());
+					def->Write(Analyzer::GetResultsSystemDefinitionsElm());
 					prevIdLength = definitionId.length();					
 				}
    			}
@@ -271,13 +292,14 @@ void Analyzer::Run(StringVector* definitionIds) {
 		VariableValue::ClearCache();
 
 		this->FinializeResultsDocument();
+
 	} else {
-		string logMessage = "\n    No definitions found! \n";
+		string logMessage = "\n    No definitions found in the input oval-definitions document! \n";
 		cout << logMessage;
 		Log::UnalteredMessage(logMessage);
-		throw AnalyzerException("");
 	}
 }
+
 void Analyzer::PrintResults() {
 
 	///////////////////////////////////////////////////////////////////////////
@@ -289,10 +311,10 @@ void Analyzer::PrintResults() {
 	///////////////////////////////////////////////////////////////////////////
 	//	Print the vulnerable results
 	///////////////////////////////////////////////////////////////////////////
-	cout << "    OVAL Id                            Result" << endl;
-	cout << "    --------------------------------------------------" << endl;
-	Log::UnalteredMessage("    OVAL Id                           Result\n");
-	Log::UnalteredMessage("    --------------------------------------------------\n");
+	cout << "    OVAL Id                                 Result" << endl;
+	cout << "    -------------------------------------------------------" << endl;
+	Log::UnalteredMessage("    OVAL Id                                 Result\n");
+	Log::UnalteredMessage("    -------------------------------------------------------\n");
 
 	// print each result value 
 	StringPairVector::iterator it;
@@ -331,9 +353,9 @@ void Analyzer::PrintResults() {
 		Log::UnalteredMessage(resultStr + "\n");
 	}
 
-	cout << "    --------------------------------------------------\n" << endl;
+	cout << "    -------------------------------------------------------\n" << endl;
 	cout << "\n ** finished evaluating OVAL definitions.\n" << endl;	
-	Log::UnalteredMessage("    --------------------------------------------------\n\n");
+	Log::UnalteredMessage("    -------------------------------------------------------\n\n");
 	Log::UnalteredMessage("\n ** finished evaluating OVAL definitions.\n\n");
 
 }
@@ -341,7 +363,7 @@ void Analyzer::PrintResults() {
 string Analyzer::ResultPairToStr(StringPair* pair) {
 
 	string resultStr = "    ";
-	resultStr.append(Common::PadString(pair->first, 35));
+	resultStr.append(Common::PadString(pair->first, 40));
 	resultStr.append(Common::PadString(pair->second, 15));
 
 	return resultStr;
@@ -368,8 +390,9 @@ void Analyzer::InitResultsDocument() {
 	DOMElement *generatorElm = XmlCommon::AddChildElement(DocumentManager::GetResultDocument(), ovalResultsElm, "generator");
 	XmlCommon::AddChildElement(DocumentManager::GetResultDocument(), generatorElm, "oval:product_name", "OVAL Definition Interpreter");
 	XmlCommon::AddChildElement(DocumentManager::GetResultDocument(), generatorElm, "oval:product_version", Version::GetVersion() + " Build: " + Version::GetBuild());
-	XmlCommon::AddChildElement(DocumentManager::GetResultDocument(), generatorElm, "oval:schema_version", "5.3");
+	XmlCommon::AddChildElement(DocumentManager::GetResultDocument(), generatorElm, "oval:schema_version", "5.4");
 	XmlCommon::AddChildElement(DocumentManager::GetResultDocument(), generatorElm, "oval:timestamp", Common::GetTimeStamp());
+	XmlCommon::AddChildElement(DocumentManager::GetResultDocument(), generatorElm, "vendor", Version::GetVendor());
 
 	// add the directives
 	DOMElement *directivesElm = XmlCommon::AddChildElement(DocumentManager::GetResultDocument(), ovalResultsElm, "directives");
@@ -398,25 +421,9 @@ void Analyzer::InitResultsDocument() {
 	XmlCommon::AddAttribute(defNotAppElm, "reported", "true");
 	XmlCommon::AddAttribute(defNotAppElm, "content", "full");
 
-	// used to add the definition doc here
-
 	// add the results element
 	DOMElement *resultsElm = XmlCommon::AddChildElement(DocumentManager::GetResultDocument(), ovalResultsElm, "results");
-	this->resultsElm = resultsElm;
-
-	// add the system element
-	DOMElement *systemElm = XmlCommon::AddChildElement(DocumentManager::GetResultDocument(), resultsElm, "system");
-	this->resultsSystemElm = systemElm;
-
-	// add the definitions element
-	DOMElement *definitionsElm = XmlCommon::AddChildElement(DocumentManager::GetResultDocument(), systemElm, "definitions");
-	Analyzer::definitionsElm = definitionsElm;
-
-	// add the tests element
-	DOMElement *testsElm = XmlCommon::AddChildElement(DocumentManager::GetResultDocument(), systemElm, "tests");
-	Analyzer::testsElm = testsElm;
-
-	// used to add the sc documnet here
+	Analyzer::resultsElm = resultsElm;
 }
 
 void Analyzer::FinializeResultsDocument() {
@@ -426,9 +433,9 @@ void Analyzer::FinializeResultsDocument() {
 	DOMElement* definitionNode = (DOMElement*)DocumentManager::GetResultDocument()->importNode(DocumentManager::GetDefinitionDocument()->getDocumentElement(), true);
 	ovalResultsElm->insertBefore(definitionNode, this->resultsElm);
 	// need to clean up the attributes on the oval_definitiosn element.
-	// copy all namespaces the documnet root
-	// add all schema locations to the docuement root.
-	// leave only the xmlns attribute on the element to seet the dedfault ns for all child elements.
+	// copy all namespaces the document root
+	// add all schema locations to the document root.
+	// leave only the xmlns attribute on the element to set the default ns for all child elements.
 	XmlCommon::CopyNamespaces(DocumentManager::GetDefinitionDocument(), DocumentManager::GetResultDocument());
 	XmlCommon::CopySchemaLocation(DocumentManager::GetDefinitionDocument(), DocumentManager::GetResultDocument());
 	XmlCommon::RemoveAttributes(definitionNode);
@@ -437,10 +444,10 @@ void Analyzer::FinializeResultsDocument() {
 	// add the oval_system characteristics element
 	DOMElement* scNode = (DOMElement*)DocumentManager::GetResultDocument()->importNode(DocumentManager::GetSystemCharacterisitcsDocument()->getDocumentElement(), true);
 	this->resultsSystemElm->appendChild(scNode);
-	// need to clean up the attributes on the oval_definitiosn element.
-	// copy all namespaces the documnet root
-	// add all schema locations to the docuement root.
-	// leave only the xmlns attribute on the element to seet the dedfault ns for all child elements.
+	// need to clean up the attributes on the oval_definitions element.
+	// copy all namespaces the document root
+	// add all schema locations to the document root.
+	// leave only the xmlns attribute on the element to seet the default ns for all child elements.
 	XmlCommon::CopyNamespaces(DocumentManager::GetSystemCharacterisitcsDocument(), DocumentManager::GetResultDocument());
 	XmlCommon::CopySchemaLocation(DocumentManager::GetSystemCharacterisitcsDocument(), DocumentManager::GetResultDocument());
 	XmlCommon::RemoveAttributes(scNode);
@@ -457,5 +464,3 @@ AnalyzerException::AnalyzerException(string errMsgIn, int severity, Exception* e
 AnalyzerException::~AnalyzerException() {
 
 }
-
-
