@@ -1,8 +1,8 @@
 //
-// $Id: CollectedObject.cpp 4138 2007-06-22 14:12:43Z bakerj $
+// $Id: CollectedObject.cpp 4659 2008-01-17 22:26:26Z bakerj $
 //
 //****************************************************************************************//
-// Copyright (c) 2002-2007, The MITRE Corporation
+// Copyright (c) 2002-2008, The MITRE Corporation
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without modification, are
@@ -41,7 +41,7 @@ CollectedObject::CollectedObject(string id, string comment, int version, int var
 	// -----------------------------------------------------------------------
 	//	Abstract
 	//
-	//	Create a compelete CollectedObject object
+	//	Create a complete CollectedObject object
 	//
 	// -----------------------------------------------------------------------
 
@@ -125,7 +125,7 @@ CollectedObject* CollectedObject::CreateNotSupported(AbsObject* absObject) {
 	// -----------------------------------------------------------------------
 	//	Abstract
 	//
-	//	Create a new ColelctedObject based on the specified AbsObject
+	//	Create a new CollectedObject based on the specified AbsObject
 	//	Set it up as not supported.
 	//
 	// -----------------------------------------------------------------------
@@ -153,6 +153,7 @@ void CollectedObject::WriteCollectedObjects() {
 		CollectedObject* obj = iterator->second;
 		obj->Write(DocumentManager::GetSystemCharacterisitcsDocument(), AbsDataCollector::Instance()->GetSCCollectedObjectsElm());
 		delete obj;
+		obj = NULL;
 	}
 	
 	CollectedObject::collectedObjectsMap.clear();
@@ -355,26 +356,84 @@ void CollectedObject::SetVersion(int version) {
 }
 
 void CollectedObject::AppendOvalMessage(OvalMessage* message) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Add an message to the end of the messages vector
-	//
-	// -----------------------------------------------------------------------
 
 	this->messages.push_back(message);
 }
 
-void CollectedObject::AppendReference(Item* reference) {
-	// -----------------------------------------------------------------------
-	//	Abstract
+void CollectedObject::AppendReferencesAndComputeFlag(ItemVector* references) {
+
+	this->AppendReferences(references);
+
+	// compute the flag value for the collected obj beased on the set of items
+
+	// get counts of each status value set on the items in the list
+	unsigned int errorCnt = 0;
+	unsigned int existsCnt = 0;
+	unsigned int doesNotExistCnt = 0;
+	unsigned int notCollectedCnt = 0;
+
+	// now loop through the items and examine their status values.
+	ItemVector::iterator iterator;
+	for(iterator = references->begin(); iterator != references->end(); iterator++) {
+		Item* tmp = (*iterator);
+
+		if(tmp->GetStatus() == OvalEnum::STATUS_ERROR) {
+			errorCnt++;
+		} else if(tmp->GetStatus() == OvalEnum::STATUS_EXISTS) {
+			existsCnt++;
+		} else if(tmp->GetStatus() == OvalEnum::STATUS_DOES_NOT_EXIST) {
+			doesNotExistCnt++;
+		} else if(tmp->GetStatus() == OvalEnum::STATUS_NOT_COLLECTED) {
+			notCollectedCnt++;
+		}
+	}
+
+	// status values -  error, exists, does not exist, not collected
+
+	// Flag == error
+	// if the set contains any item marked as error
+	if(errorCnt > 0) {
+		this->SetFlag(OvalEnum::FLAG_ERROR);
+
+	// Flag == complete
+	// if the set contains
+	//   at least one item with a status of exists and
+	//   no items with a status of not collected and
+	//   no items with a status of error
+	} else if(errorCnt == 0 && notCollectedCnt == 0 && existsCnt > 0) {
+		this->SetFlag(OvalEnum::FLAG_COMPLETE);
+
+	// Flag == incomplete
+	// if the set contains
+	//  one ore more items with a status of not collected
+	//  no items with a status of error
+	} else if(errorCnt == 0 && notCollectedCnt > 0) {
+		this->SetFlag(OvalEnum::FLAG_INCOMPLETE);
+
+	// Flag == does not exist
+	// if the set is empty, or the set contains
+	//   no items with a status of exists
+	//   no items with a status of error
+	//   no items with a status of not collected
+	//   all items have a status of does not exist
+	} else if((errorCnt == 0 && notCollectedCnt == 0 && existsCnt == 0) || references->size() == 0) {
+		this->SetFlag(OvalEnum::FLAG_DOES_NOT_EXIST);
+
+	// Flag == not collected
+	// if all items in the set have a status of not collected and
+	// the set has atleast one item
 	//
-	//	Add an reference to the end of the references vector
-	//	Assign the items id before adding to vector.
+	// NOTE: This condition is currently handled elsewhere in 
+	// the code see: CollectedObject* AbsObjectCollector::ProcessObject(Object* object)
 	//
-	// -----------------------------------------------------------------------
+	} else if(references->size() == notCollectedCnt && notCollectedCnt > 0) {
+		this->SetFlag(OvalEnum::FLAG_DOES_NOT_EXIST);
+	}
+
+	// Flag == not applicable
+	// 
+	// handled elsewhere in the code see: CollectedObject* AbsObjectCollector::ProcessObject(Object* object)
 
-	this->references.push_back(reference);
 }
 
 void CollectedObject::AppendReferences(ItemVector* references) {
@@ -396,7 +455,7 @@ void CollectedObject::AppendVariableValue(VariableValue* variableValue) {
 	// -----------------------------------------------------------------------
 	//	Abstract
 	//
-	//	Add a varaible value to the end of teh variable values vector
+	//	Add a varaible value to the end of the variable values vector
 	//
 	// -----------------------------------------------------------------------
 
@@ -500,7 +559,7 @@ bool CollectedObject::IsWritten(StringPairVector* varIdValuePairs, string varId,
 	// -----------------------------------------------------------------------
 	//	Abstract
 	//
-	//	return true if the specifed variable value has not alredy been written
+	//	return true if the specified variable value has not already been written
 	//  must check the pair of var id and value.
 	// -----------------------------------------------------------------------
 
@@ -524,7 +583,7 @@ bool CollectedObject::IsWritten(IntVector* itemIds, int itemId) {
 	// -----------------------------------------------------------------------
 	//	Abstract
 	//
-	//	return true if the specifed item id has already been 
+	//	return true if the specified item id has already been 
 	//	added as a reference
 	// -----------------------------------------------------------------------
 
@@ -559,24 +618,10 @@ void CollectedObject::Cache(CollectedObject* collectedObject) {
 //								CollectedObjectException Class							  //	
 //****************************************************************************************//
 CollectedObjectException::CollectedObjectException(string errMsgIn, int severity, Exception* ex) : Exception(errMsgIn, severity, ex) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Set the error message and then set the severity to ERROR_FATAL. This is 
-	//	done with the explicit call to the Exception class constructor that 
-	//	takes a single string param.
-	//
-	// -----------------------------------------------------------------------
 
 }
 
 CollectedObjectException::~CollectedObjectException() {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Do nothing for now
-	//
-	// -----------------------------------------------------------------------
 
 }
 
