@@ -1,8 +1,7 @@
 //
-// $Id: EntityComparator.cpp 4200 2007-06-26 20:38:26Z bakerj $
 //
 //****************************************************************************************//
-// Copyright (c) 2002-2007, The MITRE Corporation
+// Copyright (c) 2002-2008, The MITRE Corporation
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without modification, are
@@ -58,12 +57,7 @@ OvalEnum::ResultEnumeration EntityComparator::CompareBinary(OvalEnum::Operation
 }
 
 OvalEnum::ResultEnumeration EntityComparator::CompareBoolean(OvalEnum::Operation op, string defValue, string scValue) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	compare two boolean values based on the specified operation
-	//  can have the following values: {true, false, 1, 0)
-	// -----------------------------------------------------------------------
+
 	OvalEnum::ResultEnumeration result = OvalEnum::RESULT_ERROR;
 
 	bool defBoolValue = false;
@@ -71,10 +65,18 @@ OvalEnum::ResultEnumeration EntityComparator::CompareBoolean(OvalEnum::Operation
 
 	if(defValue.compare("true") == 0 || defValue.compare("1") == 0) {
 		defBoolValue = true;
+	} else if(defValue.compare("false") == 0 || defValue.compare("0") == 0) {
+		defBoolValue = false;
+	} else {
+		throw Exception("Error: Invalid boolean value on definition entity. " + defValue);
 	}
 
 	if(scValue.compare("true") == 0 || scValue.compare("1") == 0) {
 		scBoolValue = true;
+	} else if(scValue.compare("false") == 0 || scValue.compare("0") == 0) {
+		scBoolValue = false;
+	} else {
+		throw Exception("Error: Invalid boolean value on system characteristics item entity. " + scValue);
 	}
 
 	if(op == OvalEnum::OPERATION_EQUALS) {
@@ -97,12 +99,7 @@ OvalEnum::ResultEnumeration EntityComparator::CompareBoolean(OvalEnum::Operation
 }
 
 OvalEnum::ResultEnumeration EntityComparator::CompareEvrString(OvalEnum::Operation op, string defValue, string scValue) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Compare two evr string values based on the specified operation
-	//  This code motivated (strongly) by librpm's rpmdsCompare().
-	// -----------------------------------------------------------------------
+	
 	OvalEnum::ResultEnumeration result = OvalEnum::RESULT_ERROR;
 
 	string installedEpochStr = EntityComparator::GetEpochFromEVR(scValue);
@@ -114,19 +111,19 @@ OvalEnum::ResultEnumeration EntityComparator::CompareEvrString(OvalEnum::Operati
 
 	int sense = 1; // default to later
 
-	if(installedEpochStr.compare(defEpochStr) == 0) {
+	int defEpochInt = atoi(defEpochStr.c_str());
+	int installedEpochInt = atoi(installedEpochStr.c_str());
+
+	if(defEpochInt == installedEpochInt) {
+
 		sense = rpmvercmp(installedVersionStr.c_str(),defVersionStr.c_str());
 		if (sense == 0) {
 			sense = rpmvercmp(installedReleaseStr.c_str(),defReleaseStr.c_str());
 		}
 	} else {
-		int installedEpochInt = atoi(defEpochStr.c_str());
-		int defEpochInt = atoi(installedEpochStr.c_str());
-		if(installedEpochInt > 0) {
-			sense = 1;
-		} else if(defEpochInt > 0) {
-			sense = -1;
-		}
+		
+		sense = (installedEpochInt > defEpochInt)? 1 : -1;
+
 	}
 
 	// convert sense value to a result based on operator
@@ -293,19 +290,13 @@ int EntityComparator::xisupper(int c) {
 	return (c >= 'A' && c <= 'Z');
 }
 
-
 OvalEnum::ResultEnumeration EntityComparator::CompareFloat(OvalEnum::Operation op, string defValue, string scValue) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	compare two float values based on the specified operation
-	// -----------------------------------------------------------------------
 
 	OvalEnum::ResultEnumeration result = OvalEnum::RESULT_ERROR;
 
-	// convert the strings to integers
+	// convert the strings to doubles
 	double defDouble = atof(defValue.c_str());
-	double scDouble = atoi(scValue.c_str());
+	double scDouble = atof(scValue.c_str());
 
 	if(op == OvalEnum::OPERATION_EQUALS) {
 		if(scDouble == defDouble) {
@@ -315,9 +306,9 @@ OvalEnum::ResultEnumeration EntityComparator::CompareFloat(OvalEnum::Operation o
 		}
 	} else if(op == OvalEnum::OPERATION_NOT_EQUAL) {
 		if(scDouble != defDouble) {
-			result = OvalEnum::RESULT_FALSE;
-		} else {
 			result = OvalEnum::RESULT_TRUE;
+		} else {
+			result = OvalEnum::RESULT_FALSE;
 		}
 	} else if(op == OvalEnum::OPERATION_LESS_THAN) {
 		
@@ -361,11 +352,6 @@ OvalEnum::ResultEnumeration EntityComparator::CompareIosVersion(OvalEnum::Operat
 	return result;
 }
 OvalEnum::ResultEnumeration EntityComparator::CompareInteger(OvalEnum::Operation op, string defValue, string scValue) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	compare two integer values based on the specified operation
-	// -----------------------------------------------------------------------
 
 	OvalEnum::ResultEnumeration result = OvalEnum::RESULT_ERROR;
 
@@ -381,9 +367,9 @@ OvalEnum::ResultEnumeration EntityComparator::CompareInteger(OvalEnum::Operation
 		}
 	} else if(op == OvalEnum::OPERATION_NOT_EQUAL) {
 		if(scInt != defInt) {
-			result = OvalEnum::RESULT_FALSE;
-		} else {
 			result = OvalEnum::RESULT_TRUE;
+		} else {
+			result = OvalEnum::RESULT_FALSE;
 		}
 	} else if(op == OvalEnum::OPERATION_LESS_THAN) {
 		
@@ -427,11 +413,7 @@ OvalEnum::ResultEnumeration EntityComparator::CompareInteger(OvalEnum::Operation
 }
 
 OvalEnum::ResultEnumeration EntityComparator::CompareString(OvalEnum::Operation op, string defValue, string scValue) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	compare two string values based on the specified operation
-	// -----------------------------------------------------------------------
+
 	OvalEnum::ResultEnumeration result = OvalEnum::RESULT_ERROR;
 
 	if(op == OvalEnum::OPERATION_EQUALS) {
@@ -446,30 +428,6 @@ OvalEnum::ResultEnumeration EntityComparator::CompareString(OvalEnum::Operation
 		} else {
 			result = OvalEnum::RESULT_FALSE;			
 		}
-	} else if(op == OvalEnum::OPERATION_GREATER_THAN) {
-		if(defValue.compare(scValue) < 0) {
-			result = OvalEnum::RESULT_TRUE;
-		} else {
-			result = OvalEnum::RESULT_FALSE;			
-		}
-	} else if(op == OvalEnum::OPERATION_GREATER_THAN_OR_EQUAL) {
-		if(defValue.compare(scValue) <= 0) {
-			result = OvalEnum::RESULT_TRUE;
-		} else {
-			result = OvalEnum::RESULT_FALSE;			
-		}
-	} else if(op == OvalEnum::OPERATION_LESS_THAN) {
-		if(defValue.compare(scValue) > 0) {
-			result = OvalEnum::RESULT_TRUE;
-		} else {
-			result = OvalEnum::RESULT_FALSE;			
-		}
-	} else if(op == OvalEnum::OPERATION_LESS_THAN_OR_EQUAL) {
-		if(defValue.compare(scValue) >= 0) {
-			result = OvalEnum::RESULT_TRUE;
-		} else {
-			result = OvalEnum::RESULT_FALSE;			
-		}
 	} else if(op == OvalEnum::OPERATION_PATTERN_MATCH) {
 		REGEX myRegex;
 		if(myRegex.IsMatch(defValue.c_str(), scValue.c_str())) {
@@ -485,11 +443,7 @@ OvalEnum::ResultEnumeration EntityComparator::CompareString(OvalEnum::Operation
 }
 
 OvalEnum::ResultEnumeration EntityComparator::CompareVersion(OvalEnum::Operation op, string defValue, string scValue) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	compare two version values based on the specified operation
-	// -----------------------------------------------------------------------
+
 	OvalEnum::ResultEnumeration result = OvalEnum::RESULT_ERROR;
 	
 	// Invalid ops first
@@ -597,6 +551,10 @@ OvalEnum::ResultEnumeration EntityComparator::CompareVersion(OvalEnum::Operation
 		}
 	}
 
+	// delete the vectors of integers
+	delete defValues;
+	delete scValues;
+
 	return result;
 }
 
@@ -608,7 +566,7 @@ IntVector* EntityComparator::ParseVersionStr(string versionStr) {
 	
 	IntVector* tokens = new IntVector();
 
-	unsigned int index = versionStr.find_first_not_of("0123456789");
+	size_t index = versionStr.find_first_not_of("0123456789");
 	if(index == string::npos) {
 		// no delmiter found and not an empty string.
 		// simply try to convert the versionStr to an int and return it as the only token.
@@ -635,7 +593,7 @@ IntVector* EntityComparator::ParseVersionStr(string versionStr) {
 				string tokenStr = token;
 				
 				// check that the token is an integer
-				unsigned int nonInt = tokenStr.find_first_not_of("0123456789");
+				size_t nonInt = tokenStr.find_first_not_of("0123456789");
 				if(nonInt != string::npos) {
 					if(theString != NULL) {
 						free(theString);
@@ -654,7 +612,6 @@ IntVector* EntityComparator::ParseVersionStr(string versionStr) {
 			}
 		}
 
-
 		if(theString != NULL) {
 			free(theString);
 		}
