@@ -1,8 +1,7 @@
 //
-// $Id: Item.cpp 3897 2007-04-18 14:25:34Z bakerj $
 //
 //****************************************************************************************//
-// Copyright (c) 2002-2007, The MITRE Corporation
+// Copyright (c) 2002-2008, The MITRE Corporation
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without modification, are
@@ -32,19 +31,12 @@
 #include "Item.h"
 
 int Item::idCounter = 1;
-//ItemVector Item::processedItems;
 ItemMap Item::processedItemsMap;
 
 //****************************************************************************************//
 //									Item Class											  //	
 //****************************************************************************************//
 Item::Item(int id , string xmlns, string xmlnsAlias, string schemaLocation, OvalEnum::SCStatus status, string name) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Create a compelete Entity object
-	//
-	// -----------------------------------------------------------------------
 
 	this->SetId(id);
 	this->SetXmlns(xmlns);
@@ -56,12 +48,6 @@ Item::Item(int id , string xmlns, string xmlnsAlias, string schemaLocation, Oval
 }
 
 Item::~Item() {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	clean up all the elements
-	//
-	// -----------------------------------------------------------------------
 	
 	this->DeleteElements();
 }
@@ -70,13 +56,7 @@ Item::~Item() {
 //								 Static Public members										//
 // ***************************************************************************************	//
 int Item::AssignId() {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Assign a new Item Id
-	//
-	// -----------------------------------------------------------------------
-	
+
 	return Item::idCounter++;
 }
 
@@ -155,7 +135,7 @@ OvalMessageVector* Item::GetMessages() {
 	// -----------------------------------------------------------------------
 	//	Abstract
 	//
-	//	Set the messages field's value
+	//	Get the messages field's value
 	//
 	// -----------------------------------------------------------------------
 	return &this->messages;
@@ -330,24 +310,28 @@ bool Item::Equals(Item* item) {
 
 					// compare all object elements
 					unsigned int myElementCount = myObjElms->size();
-					unsigned int i = 0;
-					bool finished = false;
-					while(i < myElementCount && !finished) {
-						ItemEntity* myElement = (ItemEntity*)myObjElms->at(i++);
-						
-						string elmName = myElement->GetName();
-						// loop over itemObjElms to find the element with the same name
-						ItemEntityVector::iterator it;
-						for(it = itemObjElms->begin(); it != itemObjElms->end(); it++) {
-							if((*it)->GetName().compare(elmName) == 0) {
-								if(myElement->Equals((*it))) {
-									isEqual = true;
-									finished = false;
-									break;
-								} else {
-									isEqual = false;
-									finished = true;
-									break;
+					if(myElementCount == 0) {
+						isEqual = true;
+					} else {
+						unsigned int i = 0;
+						bool finished = false;
+						while(i < myElementCount && !finished) {
+							ItemEntity* myElement = (ItemEntity*)myObjElms->at(i++);
+							
+							string elmName = myElement->GetName();
+							// loop over itemObjElms to find the element with the same name
+							ItemEntityVector::iterator it;
+							for(it = itemObjElms->begin(); it != itemObjElms->end(); it++) {
+								if((*it)->GetName().compare(elmName) == 0) {
+									if(myElement->Equals((*it))) {
+										isEqual = true;
+										finished = false;
+										break;
+									} else {
+										isEqual = false;
+										finished = true;
+										break;
+									}
 								}
 							}
 						}
@@ -390,11 +374,6 @@ ItemEntityVector* Item::GetObjectElements() {
 }
 
 ItemEntityVector* Item::GetElementsByName(string elementName) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Return all elements with the specified name
-	// -----------------------------------------------------------------------
 
 	ItemEntityVector* matchingElements = new ItemEntityVector();
 
@@ -408,13 +387,8 @@ ItemEntityVector* Item::GetElementsByName(string elementName) {
 
 	return matchingElements;
 }
+
 void Item::Parse(DOMElement* scItemElm) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Parse the provided item from an sc file into an Item object
-	//
-	// -----------------------------------------------------------------------
 
 	this->SetName(XmlCommon::GetElementName(scItemElm));
 	this->SetId(atoi((XmlCommon::GetAttributeByName(scItemElm, "id")).c_str()));
@@ -444,17 +418,10 @@ void Item::Parse(DOMElement* scItemElm) {
 			}
 		}
 		index ++;
-	}
-	
+	}	
 }
 
 void Item::Write(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument* scFile, DOMElement* itemsElm) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Write this item to a sc file. Make sure the item has not already been 
-	//	written to the sc file
-	// -----------------------------------------------------------------------
 
 	if(!this->GetIsWritten()) {
 		this->SetIsWritten(true);
@@ -499,16 +466,36 @@ void Item::Write(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument* scFile, DOMElement*
 	}
 }
 
+Item* Item::GetItemById(string itemId) {
+
+	Item* item = NULL;
+	
+	// Search the cache
+	item = Item::SearchCache(atoi(itemId.c_str()));
+
+	// if not found try to parse it.
+	if(item == NULL) {
+
+		DOMElement* systemDataElm = XmlCommon::FindElement(DocumentManager::GetSystemCharacterisitcsDocument(), "system_data");
+		DOMElement* itemElm = XmlCommon::FindElementByAttribute(systemDataElm, "id", itemId);
+
+		if(itemElm == NULL) {
+			throw Exception("Unable to find specified item in system-characteristics document. Item id: " + itemId);
+		}
+
+		item = new Item();
+		item->Parse(itemElm);
+		Item::Cache(item);
+	}
+	
+	return item;
+}
+
 // ***************************************************************************************	//
 //								 Private members											//
 // ***************************************************************************************	//
 void Item::DeleteElements() {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Delete all the elements in the item entity vector.
-	//
-	// -----------------------------------------------------------------------
+	
 	ItemEntity* currentElement = NULL;
 	while(elements.size() != 0) {
 	  	currentElement = elements[elements.size()-1];
@@ -527,23 +514,9 @@ void Item::DeleteElements() {
 }
 
 Item* Item::SearchCache(int id) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	search the cache of Items for the specifed Item. 
-	//	return NULL if not found
-	//
-	// -----------------------------------------------------------------------
 
 	Item* cachedItem = NULL;
 
-	/*ItemVector::iterator iterator;
-	for(iterator = Item::processedItems.begin(); iterator != Item::processedItems.end(); iterator++) {
-		if((*iterator)->GetId() == id) {
-			cachedItem = (*iterator);
-			break;
-		}
-	}*/
 	ItemMap::iterator iterator;
 	iterator = Item::processedItemsMap.find(id);
 	if(iterator != Item::processedItemsMap.end()) {
@@ -554,20 +527,6 @@ Item* Item::SearchCache(int id) {
 }
 
 void Item::ClearCache() {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	delete all items in the cache
-	//
-	// -----------------------------------------------------------------------
-
-	//Item* item = NULL;
-	//while(Item::processedItems.size() != 0) {
-	//  	item = Item::processedItems[Item::processedItems.size()-1];
-	//  	Item::processedItems.pop_back();
-	//  	delete item;
-	//  	item = NULL;
-	//}
 
 	ItemMap::iterator iterator;
 	for(iterator = Item::processedItemsMap.begin(); iterator != Item::processedItemsMap.end(); iterator++) {
@@ -577,41 +536,20 @@ void Item::ClearCache() {
 	}
 	
 	Item::processedItemsMap.clear();
-
 }
 
 void Item::Cache(Item* item) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	cache the specified filter
-	//	TODO - do i need to add protection to this cache
-	// -----------------------------------------------------------------------
-	
+
 	Item::processedItemsMap.insert(ItemPair(item->GetId(), item));
 }
+
 //****************************************************************************************//
 //								ItemException Class										  //	
 //****************************************************************************************//
 ItemException::ItemException(string errMsgIn, int severity, Exception* ex) : Exception(errMsgIn, severity, ex) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Set the error message and then set the severity to ERROR_FATAL. This is 
-	//	done with the explicit call to the Exception class constructor that 
-	//	takes a single string param.
-	//
-	// -----------------------------------------------------------------------
 
 }
 
 ItemException::~ItemException() {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Do nothing for now
-	//
-	// -----------------------------------------------------------------------
 
 }
-
