@@ -1,8 +1,7 @@
 //
-// $Id: Main.cpp 4237 2007-06-28 16:02:55Z bakerj $
 //
 //****************************************************************************************//
-// Copyright (c) 2002-2007, The MITRE Corporation
+// Copyright (c) 2002-2008, The MITRE Corporation
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without modification, are
@@ -83,7 +82,7 @@ int main(int argc, char* argv[]) {
 	headerMessage.append("OVAL Definition Interpreter\n");
     headerMessage.append("Version: " + Version::GetVersion() + " Build: " + Version::GetBuild() +"\n");
     headerMessage.append("Build date: " + Version::GetBuildDate() + "\n");
-	headerMessage.append("Copyright (c) 2002-2007 - The MITRE Corporation\n");
+	headerMessage.append("Copyright (c) 2002-2008 - The MITRE Corporation\n");
 	headerMessage.append("----------------------------------------------------\n");
 	headerMessage.append("\n");
 	headerMessage.append(timeBuffer);
@@ -276,7 +275,7 @@ int main(int argc, char* argv[]) {
 			string result = XslCommon::ApplyXSL(Common::GetXMLfile(), Common::GetDefinitionSchematronPath());
 			// strip the xml declaration
 			if(result.compare("") != 0) {
-				unsigned int pos = result.find(">", 0);
+				size_t pos = result.find(">", 0);
 				if(pos != string::npos) {
 					result = result.substr(pos+1, result.length()-pos);
 				}
@@ -377,7 +376,18 @@ int main(int argc, char* argv[]) {
 		
 		//	run the analyzer
 		if(Common::GetLimitEvaluationToDefinitionIds()){
-			analyzer->Run(Common::GetDefinitionIds());
+			string idFile = Common::GetDefinitionIdsFile();
+
+			StringVector* ids = NULL;
+
+			if(idFile.compare("") != 0) {
+				ids = Common::ParseDefinitionIdsFile();	
+			} else {
+				ids = Common::ParseDefinitionIdsString();				
+			}				
+
+			analyzer->Run(ids);
+			
 		} else {
 			analyzer->Run();
 		}
@@ -523,7 +533,22 @@ void ProcessCommandLine(int argc, char* argv[]) {
 					Usage();
 					exit(0);
 				} else {
-					Common::SetDefinitionIds(argv[2]);
+					Common::SetDefinitionIdsString(argv[2]);
+					Common::SetLimitEvaluationToDefinitionIds(true);
+					++argv;
+					--argc;
+				}
+
+				break;
+
+			// **********  file containing definition ids to evaluate  ********** //
+			case 'f':
+
+				if ((argc < 3) || (argv[2][0] == '-')) {
+					Usage();
+					exit(0);
+				} else {
+					Common::SetDefinitionIdsFile(argv[2]);
 					Common::SetLimitEvaluationToDefinitionIds(true);
 					++argv;
 					--argc;
@@ -685,29 +710,40 @@ void Usage() {
 	//------------------------------------------------------------------------------------//
 
 	cout << endl;
-	cout << "Command Line: >ovaldi [option] MD5Hash" << endl;
+	cout << "Command Line: ovaldi [options] MD5Hash" << endl;
 	cout << endl;
 	cout << "Options:" << endl;
-	cout << "   -h           = show options available from command line" << endl;
+	cout << "   -h           = show options available from command line." << endl;
 	cout << "\n";
-	cout << "   -o <string>  = path to the oval-definitions xml file DEFAULT=\"definitions.xml\"" << endl;
-	cout << "   -d <string>  = save data to the specified XML file DEFAULT=\"system-characteristics.xml\"" << endl;
-	cout << "   -r <string>  = save results to the specified XML file DEFAULT=\"oval-results.xml\"" << endl;
-	cout << "   -v <string>  = get external variable values from the specified XML file. DEFAULT=\"external-variables.xml\"" << endl;
+	
+	cout << "Definition Evaluation Options:" << endl;
+	cout << "   -o <string>  = path to the oval-definitions xml file. DEFAULT=\"definitions.xml\"" << endl;
+	cout << "   -v <string>  = path to external variable values file. DEFAULT=\"external-variables.xml\"" << endl;
 	cout << "   -e <string>  = evaluate the specified list of definitions. Supply definition ids as a comma seperated list like: oval:com.example:def:123" << endl;
+	cout << "   -f <string>  = path to a file containing a list of definitions to be evaluated. The file must comply with the evaluation-id schema." << endl;
 	cout << "\n";
-	cout << "   -i <string>  = use data from input System Characteristics file" << endl;
+	
+	cout << "Input Validation Options:" << endl;
+	cout << "   -m           = do not verify the oval-definitions file with an MD5 hash." << endl;
+	cout << "   -n           = perform Schematron validation of the oval-definitions file." << endl;
+	cout << "   -c <string>  = path to xsl for oval-definitions Schematron validation. DEFAULT=\"oval-definitions-schematron.xsl\"" << endl;
 	cout << "\n";
-	cout << "   -n           = perform Schematron validation of the oval-defiitions file." << endl;
-	cout << "   -c <string>  = use the specified xsl for oval-definitions Schematron validation. DEFAULT=\"oval-definitions-schematron.xsl\"" << endl;
-	cout << "\n";
-	cout << "   -m           = do not verify the oval.xml file with an MD5 hash" << endl;
-	cout << "   -p           = print all information and error messages" << endl;
+
+	cout << "Data Collection Options:" << endl;
+	cout << "   -i <string>  = path to input System Characteristics file. Evaluation will be based on the contents of the file." << endl;
 	cout << "\n";
+
+	cout << "Result Output Options:" << endl;	
+	cout << "   -d <string>  = save data to the specified XML file. DEFAULT=\"system-characteristics.xml\"" << endl;	
+	cout << "   -r <string>  = save results to the specified XML file. DEFAULT=\"oval-results.xml\"" << endl;
 	cout << "   -s           = do not apply a stylesheet to the results xml." << endl;
 	cout << "   -t <string>  = apply the sepcified xsl to the results xml. DEFAULT=\"results_to_html.xsl\"" << endl;
 	cout << "   -x <string>  = output xsl transform results to the specified file. DEFAULT=\"results.html\"" << endl;
 	cout << "\n";
-	cout << "   -z           = return md5 of current definitions.xml" << endl;
+	
+	cout << "Other Options:" << endl;
+	//cout << "   -l <string>  = save log to the specified file DEFAULT=\"ovaldi.log\"" << endl;
+	cout << "   -p           = print all information and error messages." << endl;
+	cout << "   -z           = return md5 of current oval-definitions file." << endl;
 	cout << endl;
 }
