@@ -1,8 +1,8 @@
 //
-// $Id: Test.cpp 4142 2007-06-22 15:37:13Z bakerj $
+// $Id: Test.cpp 4608 2008-01-04 18:03:02Z bakerj $
 //
 //****************************************************************************************//
-// Copyright (c) 2002-2007, The MITRE Corporation
+// Copyright (c) 2002-2008, The MITRE Corporation
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without modification, are
@@ -31,7 +31,6 @@
 
 #include "Test.h"
 
-//TestVector Test::processedTests;
 TestMap Test::processedTestsMap;
 //****************************************************************************************//
 //								Test Class												  //	
@@ -110,178 +109,82 @@ void Test::AppendTestedVariable(VariableValue* testedVariable) {
 }
 
 string Test::GetId() {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Return the id field's value
-	//
-	// -----------------------------------------------------------------------
-
+	
 	return this->id;
 }
 
 void Test::SetId(string id) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Return the id field's value
-	//
-	// -----------------------------------------------------------------------
 
 	this->id = id;
 }
 
 string Test::GetObjectId() {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Return the objectId field's value
-	//
-	// -----------------------------------------------------------------------
 
 	return this->objectId;
 }
 
 void Test::SetObjectId(string objectId) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Return the objectId field's value
-	//
-	// -----------------------------------------------------------------------
 
 	this->objectId = objectId;
 }
 
 
 string Test::GetStateId() {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Return the stateId field's value
-	//
-	// -----------------------------------------------------------------------
 
 	return this->stateId;
 }
 
 void Test::SetStateId(string stateId) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Return the stateId field's value
-	//
-	// -----------------------------------------------------------------------
 
 	this->stateId = stateId;
 }
 
 OvalEnum::ResultEnumeration Test::GetResult() {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Return the result field's value
-	//
-	// -----------------------------------------------------------------------
 
 	return this->result;
 }
 
 void Test::SetResult(OvalEnum::ResultEnumeration result) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Return the result field's value
-	//
-	// -----------------------------------------------------------------------
 
 	this->result = result;
 }
 
 int Test::GetVariableInstance() {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Return the variableInstance field's value
-	//
-	// -----------------------------------------------------------------------
 
 	return this->variableInstance;
 }
 
 void Test::SetVariableInstance(int variableInstance) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Return the variableInstance field's value
-	//
-	// -----------------------------------------------------------------------
 
 	this->variableInstance = variableInstance;
 }
 
 int Test::GetVersion() {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Return the version field's value
-	//
-	// -----------------------------------------------------------------------
 
 	return this->version;
 }
 
 void Test::SetVersion(int version) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Return the version field's value
-	//
-	// -----------------------------------------------------------------------
 
 	this->version = version;
 }
 
 bool Test::GetWritten() {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Return the written field's value
-	//
-	// -----------------------------------------------------------------------
 
 	return this->written;
 }
 
 void Test::SetWritten(bool written) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Return the written field's value
-	//
-	// -----------------------------------------------------------------------
 
 	this->written = written;
 }
 
 bool Test::GetAnalyzed() {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Return the analyzed field's value
-	//
-	// -----------------------------------------------------------------------
 
 	return this->analyzed;
 }
 
 void Test::SetAnalyzed(bool analyzed) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	Return the analyzed field's value
-	//
-	// -----------------------------------------------------------------------
 
 	this->analyzed = analyzed;
 }
@@ -312,14 +215,6 @@ void Test::ClearCache() {
 }
 
 void Test::Write(DOMElement* parentElm) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	writes a Test element
-	//	calls testedObject->Write() 
-	//	calls testedVariable->Write() for each tested var
-	//
-	// -----------------------------------------------------------------------
 
 	if(!this->GetWritten()) {
 		this->SetWritten(true);
@@ -371,13 +266,6 @@ void Test::Write(DOMElement* parentElm) {
 }
 
 void Test::Parse(DOMElement* testElm) {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	parses Test elm to a Test obj
-	//	calls Criteria->Parse() on the Criteria elm
-	//
-	// -----------------------------------------------------------------------
 
 	// get id
 	string id = XmlCommon::GetAttributeByName(testElm, "id");
@@ -409,33 +297,16 @@ void Test::Parse(DOMElement* testElm) {
 		this->SetStateId(stateId);
 	}
 
-	Test::processedTestsMap.insert(TestPair(this->GetId(), this));
+	Test::Cache(this);
 }
 
 OvalEnum::ResultEnumeration Test::Analyze() {
-	// -----------------------------------------------------------------------
-	//	Abstract
-	//
-	//	make sure not previously analyzed.
-	//	get the collected object from the sc file by its id
-	//		- check flag on collected object against check attribute on test
-	//		- copy variable values from collected object to test as tested_variables
-	//			this is just a matter of passing the variable_value element in the colelcted obj
-	//			to the testedVaraible->Parse() method.
-	//	if collected obj not ok stop
-	//	parse all referencesin the colelcted object into Items
-	//	get the state by its id and parse
-	//	analyze each item against the state
-	//	combine the result of each item with the check attribute
-	//	saves and returns the result
-	//
-	// -----------------------------------------------------------------------
 
 	if(!this->GetAnalyzed()) {
 
 		// Does the test have a object ref?
 		if(this->GetObjectId().compare("") == 0) {
-			// Assumes it is only unknowns tests that do not have an object specifier and sets result to unknown
+			// Assumes it is only unknown tests that do not have an object specifier and sets result to unknown
 			this->SetResult(OvalEnum::RESULT_UNKNOWN);
 		} else {
 			// get the collected object from the sc file
@@ -469,14 +340,17 @@ OvalEnum::ResultEnumeration Test::Analyze() {
 							
 							// create a new tested item
 						    TestedItem* testedItem = new TestedItem();
-							Item* item = Item::SearchCache(atoi(itemId.c_str()));
-							if(item != NULL) {                          
-								testedItem->SetItem(item);
-							} else {
-								// get the item elm in the sc file
-								DOMElement* itemElm = XmlCommon::FindElementByAttribute(DocumentManager::GetSystemCharacterisitcsDocument()->getDocumentElement(), "id", itemId);
-                                testedItem->Parse(itemElm);
-							}
+							Item* item = Item::GetItemById(itemId);
+							testedItem->SetItem(item);
+
+							//Item* item = Item::SearchCache(atoi(itemId.c_str()));
+							//if(item != NULL) {                          
+							//	testedItem->SetItem(item);
+							//} else {
+							//	// get the item elm in the sc file
+							//	DOMElement* itemElm = XmlCommon::FindElementByAttribute(DocumentManager::GetSystemCharacterisitcsDocument()->getDocumentElement(), "id", itemId);
+       //                         testedItem->ParseItem(itemElm);
+							//}
 							this->AppendTestedItem(testedItem);
 						
 						} else if(childName.compare("variable_value") == 0) {
@@ -609,7 +483,7 @@ OvalEnum::ResultEnumeration Test::Analyze() {
 
 				} else if(collectedObjFlag == OvalEnum::FLAG_COMPLETE) {
 					
-					OvalEnum::ResultEnumeration ovarallResult = OvalEnum::RESULT_ERROR;
+					OvalEnum::ResultEnumeration overallResult = OvalEnum::RESULT_ERROR;
 
 					// Evaluate the check existence attribute.
 					OvalEnum::ResultEnumeration existenceResult = this->EvaluateCheckExistence();
@@ -617,9 +491,9 @@ OvalEnum::ResultEnumeration Test::Analyze() {
 					// if the existence result is true evaluate the check_state attribute if there is a state
 					if(existenceResult == OvalEnum::RESULT_TRUE) {
 						if(this->GetStateId().compare("") != 0) {
-							ovarallResult = this->EvaluateCheckState();
+							overallResult = this->EvaluateCheckState();
 						} else {
-							ovarallResult = existenceResult;
+							overallResult = existenceResult;
 
 							// since we did no look at the state set the tested item result to not evaluated
 							TestedItemVector::iterator iterator;
@@ -629,7 +503,7 @@ OvalEnum::ResultEnumeration Test::Analyze() {
 						}
 
 					} else {
-						ovarallResult = existenceResult;
+						overallResult = existenceResult;
 
 						// since we did no look at the state set the tested item result to not evaluated
 						TestedItemVector::iterator iterator;
@@ -638,7 +512,7 @@ OvalEnum::ResultEnumeration Test::Analyze() {
 						}
 					}
 
-					this->SetResult(ovarallResult);
+					this->SetResult(overallResult);
 				}
 			}
 		}
@@ -688,18 +562,36 @@ OvalEnum::ResultEnumeration Test::EvaluateCheckExistence() {
 			existenceResult = OvalEnum::RESULT_TRUE;			
 		} else if(existsCount >= 0 && doesNotExistCount >= 1 && errorCount >= 0 && notCollectedCount >= 0) {
 			existenceResult = OvalEnum::RESULT_FALSE;
+		} else if(existsCount == 0 && doesNotExistCount == 0 && errorCount == 0 && notCollectedCount == 0) {
+			existenceResult = OvalEnum::RESULT_FALSE;
 		} else if(existsCount >= 0 && doesNotExistCount == 0 && errorCount >= 1 && notCollectedCount >= 0) {
 			existenceResult = OvalEnum::RESULT_ERROR;
-		} else if(existsCount >= 0 && doesNotExistCount == 0 && errorCount == 0 && notCollectedCount >= 0) {
+		} else if(existsCount >= 0 && doesNotExistCount == 0 && errorCount == 0 && notCollectedCount >= 1) {
 			existenceResult = OvalEnum::RESULT_UNKNOWN;
+		} else {
+			string msg = "Unexpected set of item statuses found while evaluating the check existence value for a test. check_existence='all_exist' Found";
+			msg.append(" exists count=" + existsCount);
+			msg.append(" does not exist count=" + doesNotExistCount);
+			msg.append(" error count=" + errorCount);
+			msg.append(" not collected count=" + notCollectedCount);
+			Log::Info(msg);
 		}
 		
 	} else if(this->GetCheckExistence() == OvalEnum::EXISTENCE_ANY_EXIST) {
 		
 		if(existsCount >= 0 && doesNotExistCount >= 0 && errorCount == 0 && notCollectedCount >= 0) {
-			existenceResult = OvalEnum::RESULT_TRUE;			
-		} else if(existsCount == 0 && doesNotExistCount == 0 && errorCount >= 1 && notCollectedCount == 0) {
+			existenceResult = OvalEnum::RESULT_TRUE;	
+		} else if(existsCount >= 1 && doesNotExistCount >= 0 && errorCount >= 1 && notCollectedCount >= 0) {
+			existenceResult = OvalEnum::RESULT_TRUE;	
+		} else if(existsCount == 0 && doesNotExistCount >= 0 && errorCount >= 1 && notCollectedCount >= 0) {
 			existenceResult = OvalEnum::RESULT_ERROR;
+		} else {
+			string msg = "Unexpected set of item statuses found while evaluating the check existence value for a test. check_existence='any_exist' Found";
+			msg.append(" exists count=" + existsCount);
+			msg.append(" does not exist count=" + doesNotExistCount);
+			msg.append(" error count=" + errorCount);
+			msg.append(" not collected count=" + notCollectedCount);
+			Log::Info(msg);
 		}
 
 	} else if(this->GetCheckExistence() == OvalEnum::EXISTENCE_AT_LEAST_ONE_EXISTS) {
@@ -712,6 +604,13 @@ OvalEnum::ResultEnumeration Test::EvaluateCheckExistence() {
 			existenceResult = OvalEnum::RESULT_ERROR;
 		} else if(existsCount == 0 && doesNotExistCount >= 0 && errorCount == 0 && notCollectedCount >= 1) {
 			existenceResult = OvalEnum::RESULT_UNKNOWN;
+		} else {
+			string msg = "Unexpected set of item statuses found while evaluating the check existence value for a test. check_existence='at_least_one_exists' Found";
+			msg.append(" exists count=" + existsCount);
+			msg.append(" does not exist count=" + doesNotExistCount);
+			msg.append(" error count=" + errorCount);
+			msg.append(" not collected count=" + notCollectedCount);
+			Log::Info(msg);
 		}
 
 	} else if(this->GetCheckExistence() == OvalEnum::EXISTENCE_NONE_EXIST) {
@@ -724,6 +623,13 @@ OvalEnum::ResultEnumeration Test::EvaluateCheckExistence() {
 			existenceResult = OvalEnum::RESULT_ERROR;
 		} else if(existsCount == 0 && doesNotExistCount >= 0 && errorCount == 0 && notCollectedCount >= 1) {
 			existenceResult = OvalEnum::RESULT_UNKNOWN;
+		} else {
+			string msg = "Unexpected set of item statuses found while evaluating the check existence value for a test. check_existence='none_exist' Found";
+			msg.append(" exists count=" + existsCount);
+			msg.append(" does not exist count=" + doesNotExistCount);
+			msg.append(" error count=" + errorCount);
+			msg.append(" not collected count=" + notCollectedCount);
+			Log::Info(msg);
 		}
 
 	} else if(this->GetCheckExistence() == OvalEnum::EXISTENCE_ONLY_ONE_EXISTS) {
@@ -734,12 +640,22 @@ OvalEnum::ResultEnumeration Test::EvaluateCheckExistence() {
 			existenceResult = OvalEnum::RESULT_FALSE;
 		} else if(existsCount == 0 && doesNotExistCount >= 0 && errorCount == 0 && notCollectedCount == 0) {
 			existenceResult = OvalEnum::RESULT_FALSE;
-		} else if((existsCount == 0 || existsCount >= 2) && doesNotExistCount >= 0 && errorCount >= 1 && notCollectedCount >= 0) {
+		} else if(existsCount == 0 && doesNotExistCount >= 0 && errorCount >= 1 && notCollectedCount >= 0) {
 			existenceResult = OvalEnum::RESULT_ERROR;
-		} else if((existsCount == 0 || existsCount >= 2) && doesNotExistCount >= 0 && errorCount == 0 && notCollectedCount >= 1) {
+		} else if(existsCount == 1 && doesNotExistCount >= 0 && errorCount >= 1 && notCollectedCount >= 0) {
+			existenceResult = OvalEnum::RESULT_ERROR;
+		} else if(existsCount == 0 && doesNotExistCount >= 0 && errorCount == 0 && notCollectedCount >= 1) {
 			existenceResult = OvalEnum::RESULT_UNKNOWN;
+		} else if(existsCount == 1 && doesNotExistCount >= 0 && errorCount == 0 && notCollectedCount >= 1) {
+			existenceResult = OvalEnum::RESULT_UNKNOWN;
+		} else {
+			string msg = "Unexpected set of item statuses found while evaluating the check existence value for a test. check_existence='only_one_exists' Found";
+			msg.append(" exists count=" + existsCount);
+			msg.append(" does not exist count=" + doesNotExistCount);
+			msg.append(" error count=" + errorCount);
+			msg.append(" not collected count=" + notCollectedCount);
+			Log::Info(msg);
 		}
-
 	}
 
 	return existenceResult;
@@ -756,19 +672,17 @@ OvalEnum::ResultEnumeration Test::EvaluateCheckState() {
 		stateResult = OvalEnum::RESULT_TRUE;
 
 	} else {
+
 		try {
-			State* tmpState =  State::SearchCache(this->GetStateId());
-			if(tmpState == NULL) {
-				// state not already cached so parse it
-				tmpState = new State(this->GetStateId());
-			}
+
+			State* state =  State::GetStateById(this->GetStateId());
 
 			// analyze each tested item
 			IntVector results;
 			TestedItemVector::iterator iterator;
 			for(iterator = this->GetTestedItems()->begin(); iterator != this->GetTestedItems()->end(); iterator++) {
 				OvalEnum::ResultEnumeration tmpResult;
-				tmpResult = tmpState->Analyze((*iterator)->GetItem());
+				tmpResult = state->Analyze((*iterator)->GetItem());
 				(*iterator)->SetResult(tmpResult);
 				results.push_back(tmpResult);
 			}
@@ -784,3 +698,32 @@ OvalEnum::ResultEnumeration Test::EvaluateCheckState() {
 
 	return stateResult;
 }
+
+void Test::Cache(Test* test) {
+
+	Test::processedTestsMap.insert(TestPair(test->GetId(), test));
+}
+
+Test* Test::GetTestById(string testId) {
+
+	Test* test = NULL;
+	
+	// Search the cache
+	test = Test::SearchCache(testId);
+
+	// if not found try to parse it.
+	if(test == NULL) {
+
+		DOMElement* testsElm = XmlCommon::FindElement(DocumentManager::GetDefinitionDocument(), "tests");
+		DOMElement* testElm = XmlCommon::FindElementByAttribute(testsElm, "id", testId);
+
+		if(testElm == NULL) {
+			throw Exception("Unable to find specified test in oval-definition document. Test id: " + testId);
+		}
+
+		test = new Test();
+		test->Parse(testElm);
+	}
+	
+	return test;
+}
