@@ -1,8 +1,7 @@
 //
-// $Id: XmlCommon.cpp 3897 2007-04-18 14:25:34Z bakerj $
 //
 //****************************************************************************************//
-// Copyright (c) 2002-2007, The MITRE Corporation
+// Copyright (c) 2002-2008, The MITRE Corporation
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without modification, are
@@ -534,7 +533,7 @@ void XmlCommon::AddSchemaLocation(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *do
 	string currentSchemaLocation = XmlCommon::GetAttributeByName(rootElm, "xsi:schemaLocation");
 
 	if(currentSchemaLocation.compare("") != 0) {
-		unsigned int pos = currentSchemaLocation.find(newSchemaLocation, 0);
+		size_t pos = currentSchemaLocation.find(newSchemaLocation, 0);
 		if(pos == string::npos) {
 			currentSchemaLocation.append(" " + newSchemaLocation);
 			XmlCommon::AddAttribute(rootElm, "xsi:schemaLocation", currentSchemaLocation);
