@@ -1,8 +1,7 @@
 //
-// $Id: InetListeningServersProbe.cpp 4193 2007-06-26 14:45:58Z bakerj $
 //
 //****************************************************************************************//
-// Copyright (c) 2002-2007, The MITRE Corporation
+// Copyright (c) 2002-2008, The MITRE Corporation
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without modification, are
@@ -737,7 +736,7 @@ NetstatResult* InetListeningServersProbe::ParentGetChildResult(int readErrh, int
   buf = (char*)malloc(sizeof(char*)*1024);
   if(buf == NULL) {
 
-    // Wait for the child process to compelete
+    // Wait for the child process to complete
     waitpid (pid, NULL, 0);
       
     // Close the pipes
@@ -794,7 +793,7 @@ NetstatResult* InetListeningServersProbe::ParentGetChildResult(int readErrh, int
     }
   }
   
-  // Wait for the child process to compelete
+  // Wait for the child process to complete
   if(waitpid (pid, NULL, 0) == -1) {
     result->errText.append("Execution of netstat in child process failed.");
     return result;
@@ -817,8 +816,8 @@ void InetListeningServersProbe::ParseNetstatResult() {
 	int preDelim;
 	int curColon;
 	int preColon;
-	unsigned int curPos;
-	unsigned int tmpPos;
+	size_t curPos;
+	size_t tmpPos;
 
 	string Protocol;
 	string LocalFullAddress;
