@@ -1,8 +1,8 @@
 //
-// $Id: AccessTokenProbe.cpp 4173 2007-06-25 18:46:05Z bakerj $
+// $Id: AccessTokenProbe.cpp 4579 2008-01-02 17:39:07Z bakerj $
 //
 //****************************************************************************************//
-// Copyright (c) 2002-2007, The MITRE Corporation
+// Copyright (c) 2002-2008, The MITRE Corporation
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without modification, are
@@ -80,12 +80,16 @@ ItemVector* AccessTokenProbe::CollectItems(Object *object) {
 		BehaviorVector::iterator iterator;
 		for(iterator = behaviors->begin(); iterator != behaviors->end(); iterator++) {
 			Behavior* behavior = (*iterator);
-			if(behavior->GetName().compare("include_group") == 0 && behavior->GetValue().compare("false") == 0) {
-				includeGroupBehavior = false;
-			} else if(behavior->GetName().compare("resolve_group") == 0 && behavior->GetValue().compare("true") == 0) {
-				resolveGroupBehavior = true;
+			if(behavior->GetName().compare("include_group") == 0) {
+				if(behavior->GetValue().compare("false") == 0) {
+					includeGroupBehavior = false;
+				}
+			} else if(behavior->GetName().compare("resolve_group") == 0) {
+				if(behavior->GetValue().compare("true") == 0) {
+					resolveGroupBehavior = true;
+				}
 			} else {
-				Log::Info("Unsupported behavior found when collecting " + object->GetId());
+				Log::Info("Unsupported behavior found when collecting " + object->GetId() + " behavior: " + behavior->GetName());
 			}
 		}		
 	}
