@@ -1,8 +1,7 @@
 //
-// $Id: FileEffectiveRightsProbe.cpp 4233 2007-06-28 14:47:36Z bakerj $
 //
 //****************************************************************************************//
-// Copyright (c) 2002-2007, The MITRE Corporation
+// Copyright (c) 2002-2008, The MITRE Corporation
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without modification, are
@@ -31,14 +30,6 @@
 
 #include "FileEffectiveRightsProbe.h"
 
-/**
-	NOTE: At this time we are not certain how to obtain all effective rights for a file.
-	The code in the FileEffectiveRightsProbe::GetEffectiveRights attempts to do this.
-	If you have any idea how to fix this issue send email to oval@mitre.org.
-
-	Thanks in advance for your comments.
-*/
-
 
 //****************************************************************************************//
 //								FileEffectiveRightsProbe Class							  //	
@@ -108,6 +99,24 @@ ItemVector* FileEffectiveRightsProbe::CollectItems(Object* object) {
 		throw ProbeException("Error: invalid operation specified on trustee_name. Found: " + OvalEnum::OperationToString(trusteeName->GetOperation()));
 	}
 
+	// support behaviors - init with defaults.
+	bool includeGroupBehavior = true;
+	bool resolveGroupBehavior = false;
+	if(object->GetBehaviors()->size() != 0) {
+		BehaviorVector* behaviors = object->GetBehaviors();
+		BehaviorVector::iterator iterator;
+		for(iterator = behaviors->begin(); iterator != behaviors->end(); iterator++) {
+			Behavior* behavior = (*iterator);
+			if(behavior->GetName().compare("include_group") == 0 && behavior->GetValue().compare("false") == 0) {
+				includeGroupBehavior = false;
+			} else if(behavior->GetName().compare("resolve_group") == 0 && behavior->GetValue().compare("true") == 0) {
+				resolveGroupBehavior = true;
+			} else {
+				Log::Info("Unsupported behavior found when collecting " + object->GetId());
+			}
+		}		
+	}
+
 	FileFinder fileFinder;
 	StringPairVector* filePaths = fileFinder.SearchFiles(path, fileName, object->GetBehaviors());
 
@@ -148,17 +157,22 @@ ItemVector* FileEffectiveRightsProbe::CollectItems(Object* object) {
 
 				try {
 
-					StringVector* trusteeNames = this->GetTrusteeNames(fp->first, fp->second, trusteeName);
-					if(trusteeNames->size() > 0) {
-						StringVector::iterator iterator;
+					//
+					// The file exists so lets get the trustees to then examine effective rights
+					//
+					StringSet* trusteeNames = this->GetTrusteeNames(fp->first, fp->second, trusteeName, resolveGroupBehavior, includeGroupBehavior);
+					if(!trusteeNames->empty()) {
+						StringSet::iterator iterator;
 						for(iterator = trusteeNames->begin(); iterator != trusteeNames->end(); iterator++) {
 							try {
 								Item* item = this->GetEffectiveRights(fp->first, fp->second, (*iterator));
 								if(item != NULL) {
 									collectedItems->push_back(item);
 								}
+							} catch (ProbeException ex) {
+								Log::Debug(ex.GetErrorMessage());
 							} catch (Exception ex) {
-								Log::Debug("Error while getting effictive rights for file. Path: " + fp->first + " Name: " + fp->second + ex.GetErrorMessage());
+								Log::Debug("Unknown error while getting effective rights for file. Path: " + fp->first + " Name: " + fp->second + " " + ex.GetErrorMessage());
 							}
 						}
 						delete trusteeNames;
@@ -227,15 +241,9 @@ Item* FileEffectiveRightsProbe::CreateItem() {
 	return item;
 }
 
-StringVector* FileEffectiveRightsProbe::GetTrusteeNames(string path, string fileName, ObjectEntity* trusteeName) {
-	// -----------------------------------------------------------------------
-	//  ABSTRACT
-	//
-	//	Return the set of trustee names that are specified by the 
-	//  ObjectEntity* trusteeName parameter.
-	// -----------------------------------------------------------------------
+StringSet* FileEffectiveRightsProbe::GetTrusteeNames(string path, string fileName, ObjectEntity* trusteeName,  bool resolveGroupBehavior, bool includeGroupBehavior) {
 
-	StringVector* trusteeNames = new StringVector();
+	StringSet* trusteeNames = new StringSet();
 	
 	// load the set of all Trustee names
 	StringVector* allTrusteeNames = WindowsCommon::GetAllTrusteeNames();
@@ -246,9 +254,9 @@ StringVector* FileEffectiveRightsProbe::GetTrusteeNames(string path, string file
 		// proceed based on operation
 		if(trusteeName->GetOperation() == OvalEnum::OPERATION_EQUALS) {
 			
-			// check that the trusee name exists
+			// check that the trustee name exists
 			if(this->TrusteeNameExists(trusteeName->GetValue(), allTrusteeNames)) {
-				trusteeNames->push_back(trusteeName->GetValue());
+				trusteeNames->insert(trusteeName->GetValue());
 			}
 
 		} else if(trusteeName->GetOperation() == OvalEnum::OPERATION_NOT_EQUAL) {
@@ -266,7 +274,7 @@ StringVector* FileEffectiveRightsProbe::GetTrusteeNames(string path, string file
 		for(it = allTrusteeNames->begin(); it != allTrusteeNames->end(); it++) {
 			tmp->SetValue((*it));
 			if(trusteeName->Analyze(tmp) == OvalEnum::RESULT_TRUE) {
-				trusteeNames->push_back((*it));
+				trusteeNames->insert((*it));
 			}
 		}
 	}
@@ -274,10 +282,44 @@ StringVector* FileEffectiveRightsProbe::GetTrusteeNames(string path, string file
 	// using common code to get these do not delete they are cached.
 	//delete allTrusteeNames;
 
-	return trusteeNames;
+	// apply the behaviors
+	StringSet* trusteeNamesSet = new StringSet();
+	if(!trusteeNames->empty()) {
+		
+		StringVector::iterator iterator;
+		StringSet::iterator it;
+		string domainStr = "";
+		string sidStr = "";
+		bool isGroup;
+
+		for(it = trusteeNames->begin(); it != trusteeNames->end(); it++) {
+			// is this a group
+			isGroup = WindowsCommon::LookUpTrusteeName(&(*it), &sidStr, &domainStr);
+
+			if(isGroup && resolveGroupBehavior) {
+
+				if(includeGroupBehavior) {
+					trusteeNamesSet->insert((*it));
+				}
+
+				// get the group members and add them to the set
+				StringVector* groupMembers = new StringVector();
+				WindowsCommon::ExpandGroup((*it), groupMembers);
+				for(iterator = groupMembers->begin(); iterator != groupMembers->end(); iterator++) {
+					trusteeNamesSet->insert((*iterator));
+				}
+				delete groupMembers;
+
+			} else {
+				trusteeNamesSet->insert((*it));
+			}
+		}
+	}
+
+	return trusteeNamesSet;
 }
 
-void FileEffectiveRightsProbe::GetMatchingTrusteeNames(string trusteeNamePattern, StringVector* allTrusteeNames, StringVector* trusteeNames, bool isRegex) {
+void FileEffectiveRightsProbe::GetMatchingTrusteeNames(string trusteeNamePattern, StringVector* allTrusteeNames, StringSet* trusteeNames, bool isRegex) {
 	// -----------------------------------------------------------------------
 	//
 	//  ABSTRACT
@@ -290,18 +332,12 @@ void FileEffectiveRightsProbe::GetMatchingTrusteeNames(string trusteeNamePattern
 	StringVector::iterator iterator;
 	for(iterator = allTrusteeNames->begin(); iterator != allTrusteeNames->end(); iterator++) {
 		if(this->IsMatch(trusteeNamePattern, (*iterator), isRegex)) {
-			trusteeNames->push_back((*iterator));
+			trusteeNames->insert((*iterator));
 		}
 	}
 }
 
 bool FileEffectiveRightsProbe::TrusteeNameExists(string trusteeName, StringVector* trusteeNames) {
-	// -----------------------------------------------------------------------
-	//
-	//  ABSTRACT
-	//
-	//	Return a true if the specified trustee name exists in the set of all names.  
-	// -----------------------------------------------------------------------
 
 	bool exists = false;
 
@@ -320,16 +356,17 @@ Item* FileEffectiveRightsProbe::GetEffectiveRights(string path, string fileName,
 	//
 	//  ABSTRACT
 	//
-	//	Return a populated item for the specified trustess on the specified file.
+	//	Return a populated item for the specified trustees on the specified file.
 	//
+	//	- Call GetNamedSecurityInfo to get a DACL Security Descriptor for the file
+	//	  http://msdn2.microsoft.com/en-us/library/aa446645.aspx
 	//	- Use provided trustee name and call LsaLookupNames to get the sid
-	//	  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/secmgmt/security/lsalookupnames.asp
-	//	- Then call GetNamedSecurityInfo to get a DACL Security Descriptor for the file
-	//	  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/secauthz/security/getnamedsecurityinfo.asp
+	//	  http://msdn2.microsoft.com/en-us/library/ms721797.aspx
 	//	- Then call GetEffectiveRightsFromAcl with the dacl and the sid found in the earlier calls
-	//	  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/secauthz/security/geteffectiverightsfromacl.asp
+	//	  http://msdn2.microsoft.com/en-us/library/aa446637.aspx
 	// -----------------------------------------------------------------------
 	
+
 	Item* item = NULL;
 
 	// build the path
@@ -343,24 +380,7 @@ Item* FileEffectiveRightsProbe::GetEffectiveRights(string path, string fileName,
 		filePath.append(fileName);
 	}
 
-	// elevate privs - this should not be needed and has no effect on the 
-	//	rights obtained by the api calls below
-	//TOKEN_PRIVILEGES tokPrivileges;
-	//HANDLE hProcess = GetCurrentProcess();
-	//HANDLE hToken = NULL;
-	//LUID secNameLUID;
-	//OpenProcessToken(hProcess, TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken);
-	//LookupPrivilegeValue(NULL, SE_SECURITY_NAME, &secNameLUID);
-	//tokPrivileges.PrivilegeCount = 1;
-	//tokPrivileges.Privileges[0].Luid = secNameLUID;
-	//tokPrivileges.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
-	//BOOL  result = AdjustTokenPrivileges(hToken, FALSE, &tokPrivileges, NULL, NULL, NULL);
-	//if(!result) {
-	//	cout << "Unable to elevate privilages" << endl;
-	//} else {
-	//	cout << "elevated" << endl;
-	//}
-	//
+	string baseErrMsg = "Error unable to get effective rights for trustee: " + trusteeName + " from dacl for file: " + filePath;
 
 	DWORD res;
 	PACL pdacl;
@@ -379,27 +399,45 @@ Item* FileEffectiveRightsProbe::GetEffectiveRights(string path, string fileName,
 
 	if (res != ERROR_SUCCESS) {
 		if (res == ERROR_FILE_NOT_FOUND) {
-			// should never get here. before calling this function the file should already have 
-			// been checked for existence.
-			throw ProbeException("Error unable locate " + filePath + " while getting trustte names."); 
+			// should never get here. 
+			// before calling this function the file should already have been checked for existence.
+			throw ProbeException( baseErrMsg + " Unable locate the specified file."); 
 		} else {
-			throw ProbeException("Error unable to retrieve a copy of the security descriptor for " + filePath + " while getting trustee names."); 
+			throw ProbeException(baseErrMsg + " Unable to retrieve a copy of the security descriptor. System error message: " + WindowsCommon::GetErrorMessage(res)); 
 		}
 	} 
 
-	// Check to see if a valid security descriptor were returned.  
+	// Check to see if a valid security descriptor was returned.  
     if ((IsValidSecurityDescriptor(pSD) == 0) || (IsValidAcl(pdacl) == 0)) {
 		LocalFree(pSD);
-		throw ProbeException("Error invalid data returned from call to GetNamedSecurityInfo(). Cannot get effective rights for file: " + filePath);
+		throw ProbeException(baseErrMsg + " Invalid data returned from call to GetNamedSecurityInfo().");
+
+		//item->SetStatus(OvalEnum::STATUS_ERROR);
+		//item->AppendMessage(new OvalMessage(baseErrMsg + " Invalid data returned from call to GetNamedSecurityInfo().", OvalEnum::LEVEL_ERROR));
+		//Log::Debug(baseErrMsg + " Invalid data returned from call to GetNamedSecurityInfo().");
+		//return item;
 	}
 
+	// the file exists so we can create the new item now.
+	item = this->CreateItem();
+	item->SetStatus(OvalEnum::STATUS_EXISTS);
+	item->AppendElement(new ItemEntity("path", path, OvalEnum::DATATYPE_STRING, true, OvalEnum::STATUS_EXISTS));
+	item->AppendElement(new ItemEntity("filename", fileName, OvalEnum::DATATYPE_STRING, true, OvalEnum::STATUS_EXISTS));
+	item->AppendElement(new ItemEntity("trustee_name", trusteeName, OvalEnum::DATATYPE_STRING, true, OvalEnum::STATUS_EXISTS));
+
+
 	// Get the sid for the trustee name
 	PSID pSid = WindowsCommon::GetSIDForTrusteeName(trusteeName);
 
 	// check the sid
 	if(!IsValidSid(pSid)) {
 		LocalFree(pSD);
-		throw ProbeException("Invalid sid for trustee: " +  trusteeName);
+		throw ProbeException(baseErrMsg + " Invalid sid found.");
+
+		//item->SetStatus(OvalEnum::STATUS_ERROR);
+		//item->AppendMessage(new OvalMessage(baseErrMsg + " Invalid sid found.", OvalEnum::LEVEL_ERROR));
+		//Log::Debug(baseErrMsg + " Invalid sid found.");
+		//return item;
 	}
 
 	// build the trustee structure
@@ -410,45 +448,57 @@ Item* FileEffectiveRightsProbe::GetEffectiveRights(string path, string fileName,
 	PACCESS_MASK accessRights = NULL;
 	accessRights = reinterpret_cast<PACCESS_MASK>(::LocalAlloc(LPTR, sizeof(PACCESS_MASK) + sizeof(ACCESS_MASK)));
 	if(accessRights == NULL) {
-		ProbeException("Out of memory");
+		throw ProbeException(baseErrMsg + " Out of memory! Unable to allocate memory for access rights.");
+
+		//item->SetStatus(OvalEnum::STATUS_ERROR);
+		//item->AppendMessage(new OvalMessage(baseErrMsg + " Out of memory! Unable to allocate memory for access rights.", OvalEnum::LEVEL_ERROR));
+		//Log::Debug(baseErrMsg + " Out of memory! Unable to allocate memory for access rights.");
+		//return item;
 	}
 
 	res = GetEffectiveRightsFromAcl(pdacl,
 									&trustee,
 									accessRights);
 	if (res != ERROR_SUCCESS) {
+		
+		string errMsg = WindowsCommon::GetErrorMessage(res);		
+
 		LocalFree(pSD);
-		throw ProbeException("Error unable to get effictive rights for trustee: " + trusteeName + " from dacl for file: " + filePath); 
+		LocalFree(accessRights);
+		
+		//item->SetStatus(OvalEnum::STATUS_ERROR);
+		//item->AppendMessage(new OvalMessage(baseErrMsg + " System error message: " + errMsg, OvalEnum::LEVEL_ERROR));
+		//Log::Debug(baseErrMsg + " System error message: " + errMsg);
+		//return item;
+
+		throw ProbeException(baseErrMsg + " System error message: " + errMsg); 		
 	} 
 		
 	// Convert access mask to binary.
-	// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/secauthz/security/access_mask_format.asp
-	int j;
+	// http://msdn2.microsoft.com/en-us/library/aa374896.aspx
 	char mask[33];
+	ZeroMemory(&mask, 33);
 
-	for (j = 0; j < 32; j++) {
+	for (int j = 0; j < 32; j++) {
 		if ((*accessRights) & (1 << j))
-			mask[31 - j] = '1';
+			mask[j] = '1';
 		else
-			mask[31 - j] = '0';
+			mask[j] = '0';
 	}
-	mask[32] = 0;
 
-	item = this->CreateItem();
-	item->SetStatus(OvalEnum::STATUS_EXISTS);
-	item->AppendElement(new ItemEntity("path", path, OvalEnum::DATATYPE_STRING, true, OvalEnum::STATUS_EXISTS));
-	item->AppendElement(new ItemEntity("filename", fileName, OvalEnum::DATATYPE_STRING, true, OvalEnum::STATUS_EXISTS));
-	item->AppendElement(new ItemEntity("trustee_name", trusteeName, OvalEnum::DATATYPE_STRING, true, OvalEnum::STATUS_EXISTS));
-
-	// removed for version 5.2 of the oval schema
-	//string domainStr = "";
-	//string sidStr = "";
-	//bool isGroup = WindowsCommon::LookUpTrusteeName(&trusteeName, &sidStr, &domainStr);
-	//item->AppendElement(new ItemEntity("trustee_domain", domainStr, OvalEnum::DATATYPE_STRING, false, OvalEnum::STATUS_EXISTS));
-	//item->AppendElement(new ItemEntity("trustee_sid", sidStr, OvalEnum::DATATYPE_STRING, false, OvalEnum::STATUS_EXISTS));
+	// need to seperatly determine if the generic bit should be set.
+	// the access mask that is returned never has the generic bits set. 
+	// Those bits can be determined by rolling up the object specific access bits
+	if((*accessRights) & FILE_GENERIC_READ)
+		mask[31] = '1';
+	if((*accessRights) & FILE_GENERIC_WRITE)
+		mask[30] = '1';
+	if((*accessRights) & FILE_GENERIC_EXECUTE)
+		mask[29] = '1';
+	if((*accessRights) & FILE_ALL_ACCESS)
+		mask[28] = '1';
 
 	// read values in the access_mask
-	// http://windowssdk.msdn.microsoft.com/en-us/library/ms717896.aspx
 	item->AppendElement(new ItemEntity("standard_delete", Common::ToString(mask[16]), OvalEnum::DATATYPE_BOOLEAN, false, OvalEnum::STATUS_EXISTS));
 	item->AppendElement(new ItemEntity("standard_read_control", Common::ToString(mask[17]), OvalEnum::DATATYPE_BOOLEAN, false, OvalEnum::STATUS_EXISTS));
 	item->AppendElement(new ItemEntity("standard_write_dac", Common::ToString(mask[18]), OvalEnum::DATATYPE_BOOLEAN, false, OvalEnum::STATUS_EXISTS));
@@ -470,6 +520,7 @@ Item* FileEffectiveRightsProbe::GetEffectiveRights(string path, string fileName,
 	item->AppendElement(new ItemEntity("file_write_attributes", Common::ToString(mask[8]), OvalEnum::DATATYPE_BOOLEAN, false, OvalEnum::STATUS_EXISTS));
 
 	LocalFree(pSD);
+	LocalFree(accessRights);
 
 	return item;
 }
