@@ -1,8 +1,7 @@
 //
-// $Id: FileProbe.cpp 4233 2007-06-28 14:47:36Z bakerj $
 //
 //****************************************************************************************//
-// Copyright (c) 2002-2007, The MITRE Corporation
+// Copyright (c) 2002-2008, The MITRE Corporation
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without modification, are
@@ -175,8 +174,6 @@ Item* FileProbe::GetFileAttributes(string path, string fileName) {
 		//
 		// SMC-AUDIT: ISSUE: should probably verify that this is a regular file before opening,
 		// instead of a virtual memory file!
-		//
-		// ARB:
 		
 		hFile = CreateFile(filePath.c_str(),			// file name
 							GENERIC_READ,				// access mode
@@ -189,28 +186,30 @@ Item* FileProbe::GetFileAttributes(string path, string fileName) {
 		if (hFile == INVALID_HANDLE_VALUE) {
 
 			DWORD errorNum = GetLastError();
+			string sysErrMsg = WindowsCommon::GetErrorMessage(errorNum);
 
-			if(errorNum == ERROR_FILE_NOT_FOUND) {
+			if(errorNum == ERROR_FILE_NOT_FOUND || errorNum == ERROR_PATH_NOT_FOUND) {
 				
-				/*errorMessage.append("(FileProbe) The file '");
+				string errorMessage = "";
+				errorMessage.append("(FileProbe) Unable to open a handle to the file: '");
 				errorMessage.append(filePath);
-				errorMessage.append("' could not be found.");*/
+				errorMessage.append("'. This error should never occur since the file has already be confirmed to exist on the system. " + sysErrMsg);
+				throw ProbeException(errorMessage);
 				
 			} else if(errorNum == ERROR_PATH_NOT_FOUND) {
 
-				/*errorMessage.append("(FileProbe) The path '");
-				errorMessage.append(path);
-				errorMessage.append("' does not exist.");*/
-
-			} else {
-				char errorCodeBuffer[33];
-				_ltoa(errorNum, errorCodeBuffer, 10);
-
 				string errorMessage = "";
 				errorMessage.append("(FileProbe) Unable to open a handle to the file: '");
 				errorMessage.append(filePath);
-				errorMessage.append("'.  Error Code - ");
-				errorMessage.append(errorCodeBuffer);
+				errorMessage.append("'. This error should never occur since the file has already be confirmed to exist on the system. " + sysErrMsg);
+				throw ProbeException(errorMessage);
+
+			} else {
+
+				string errorMessage = "";
+				errorMessage.append("(FileProbe) Unable to open a handle to the file: '");
+				errorMessage.append(filePath);
+				errorMessage.append("'.  " + sysErrMsg);
 				throw ProbeException(errorMessage);
 			}
 		}
@@ -349,9 +348,6 @@ Item* FileProbe::GetFileAttributes(string path, string fileName) {
 			errorMessage.append("'.");
 			
 			item->AppendElement(new ItemEntity("size", "", OvalEnum::DATATYPE_INTEGER, false, OvalEnum::STATUS_ERROR));
-			item->AppendElement(new ItemEntity("a_time", "", OvalEnum::DATATYPE_INTEGER, false, OvalEnum::STATUS_ERROR));
-			item->AppendElement(new ItemEntity("c_time", "", OvalEnum::DATATYPE_INTEGER, false, OvalEnum::STATUS_ERROR));
-			item->AppendElement(new ItemEntity("m_time", "", OvalEnum::DATATYPE_INTEGER, false, OvalEnum::STATUS_ERROR));
 			item->AppendMessage(new OvalMessage(errorMessage));
 			
 		} else {
@@ -361,33 +357,48 @@ Item* FileProbe::GetFileAttributes(string path, string fileName) {
 			_snprintf(buf, sizeof(buf)-1, "%ld", statusBuffer.st_size);
 			buf[sizeof(buf)-1] = '\0';
 			item->AppendElement(new ItemEntity("size", buf, OvalEnum::DATATYPE_INTEGER, false, OvalEnum::STATUS_EXISTS));
+		}
+
+
+		//////////////////////////////////////////////////////
+		/////////////////////  File Times  ///////////////////
+		//////////////////////////////////////////////////////
+		FILETIME creationTime;
+		FILETIME lastAccessTime;
+		FILETIME writeTime;
+
+		BOOL timeRes = GetFileTime(	hFile,
+									&creationTime,
+									&lastAccessTime,
+									&writeTime);
+
+		if(!timeRes) {
+
+			ItemEntity* aTime = new ItemEntity("a_time",  "", OvalEnum::DATATYPE_INTEGER, false, OvalEnum::STATUS_ERROR);
+			ItemEntity* cTime = new ItemEntity("c_time",  "", OvalEnum::DATATYPE_INTEGER, false, OvalEnum::STATUS_ERROR);
+			ItemEntity* mTime = new ItemEntity("m_time",  "", OvalEnum::DATATYPE_INTEGER, false, OvalEnum::STATUS_ERROR);
+			string lastError = WindowsCommon::GetErrorMessage(GetLastError());
+			item->AppendMessage(new OvalMessage("Unable to file times for file. " + lastError, OvalEnum::LEVEL_ERROR));
+
+		} else {
 
 			//////////////////////////////////////////////////////
 			/////////////////////  Accessed  /////////////////////
 			//////////////////////////////////////////////////////
-			ItemEntity* aTime = new ItemEntity("a_time",  Common::ToString((long)statusBuffer.st_atime), OvalEnum::DATATYPE_INTEGER, false, OvalEnum::STATUS_EXISTS);
+			ItemEntity* aTime = new ItemEntity("a_time", WindowsCommon::ToString(creationTime), OvalEnum::DATATYPE_INTEGER, false, OvalEnum::STATUS_EXISTS);
 			item->AppendElement(aTime);
-			// Strip trailing \n if found
-			if(aTime->GetValue().find_last_of("\n") == aTime->GetValue().length()-1)
-				aTime->SetValue(aTime->GetValue().substr(0, aTime->GetValue().length()-1));
 
 			//////////////////////////////////////////////////////
 			/////////////////////  Created  /////////////////////
 			//////////////////////////////////////////////////////
-			ItemEntity* cTime = new ItemEntity("c_time",  Common::ToString((long)statusBuffer.st_ctime), OvalEnum::DATATYPE_INTEGER, false, OvalEnum::STATUS_EXISTS);
+			ItemEntity* cTime = new ItemEntity("c_time", WindowsCommon::ToString(lastAccessTime), OvalEnum::DATATYPE_INTEGER, false, OvalEnum::STATUS_EXISTS);
 			item->AppendElement(cTime);
-			// Strip trailing \n if found
-			if(cTime->GetValue().find_last_of("\n") == cTime->GetValue().length()-1)
-				cTime->SetValue(cTime->GetValue().substr(0, cTime->GetValue().length()-1));
 
 			//////////////////////////////////////////////////////
 			/////////////////////  Modified  /////////////////////
 			//////////////////////////////////////////////////////
-			ItemEntity* mTime = new ItemEntity("m_time",  Common::ToString((long)statusBuffer.st_mtime), OvalEnum::DATATYPE_INTEGER, false, OvalEnum::STATUS_EXISTS);
+			ItemEntity* mTime = new ItemEntity("m_time", WindowsCommon::ToString(writeTime), OvalEnum::DATATYPE_INTEGER, false, OvalEnum::STATUS_EXISTS);
 			item->AppendElement(mTime);
-			// Strip trailing \n if found
-			if(mTime->GetValue().find_last_of("\n") == mTime->GetValue().length()-1)
-				mTime->SetValue(cTime->GetValue().substr(0, mTime->GetValue().length()-1));
 
 		}
 
