@@ -1,8 +1,8 @@
 //
-// $Id: RegistryProbe.cpp 4173 2007-06-25 18:46:05Z bakerj $
+// $Id: $
 //
 //****************************************************************************************//
-// Copyright (c) 2002-2007, The MITRE Corporation
+// Copyright (c) 2002-2008, The MITRE Corporation
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without modification, are
@@ -574,8 +574,7 @@ void RegistryProbe::KeyPatternMatch(ItemEntity* hive, string pattern, ItemEntity
 			if(ex.GetSeverity() == ERROR_WARN) {
 				string pcreMsg = "";
 				pcreMsg.append("Registry Keys Probe Warning - while searching for matching keys:\n");
-				pcreMsg.append("-----------------------------------------------------------------------\n");
-				pcreMsg.append(ex.GetErrorMessage());
+				pcreMsg.append("\t" + ex.GetErrorMessage());
 				Log::Debug(pcreMsg);
 			} else {
 				throw;
@@ -620,7 +619,11 @@ void RegistryProbe::RetrieveInfo(string hiveIn, string keyIn, string nameIn,
 					if (strlen(binaryBuf) == 1) 
 						value.append("0");
 					value.append(binaryBuf);
-					value.append(" ");
+
+					// add a space only if not at the end of the string
+					if(x < (valuelenIn - 1)) {
+						value.append(" ");
+					}
 				}
 				item->AppendElement(new ItemEntity("value",  value, OvalEnum::DATATYPE_BINARY, false, OvalEnum::STATUS_EXISTS));
 
@@ -685,7 +688,7 @@ void RegistryProbe::RetrieveInfo(string hiveIn, string keyIn, string nameIn,
 
 				string strValue = "";
 				bool addedValue = false;
-				for (DWORD x=0; x<(valuelenIn-2); x++) {
+				for (DWORD x=0; x<=(valuelenIn-2); x++) {
 
 					ZeroMemory(multiszBuf, sizeof(multiszBuf));
 					_snprintf(multiszBuf, sizeof(multiszBuf)-1, "%C", valueIn[x]);
@@ -710,6 +713,18 @@ void RegistryProbe::RetrieveInfo(string hiveIn, string keyIn, string nameIn,
 			{
 				item->AppendElement(new ItemEntity("type",  "reg_sz", OvalEnum::DATATYPE_STRING, false, OvalEnum::STATUS_EXISTS));
 
+				/*
+				if the string is empty, sometimes the valuelenIn returned by RegQueryValueEx
+				when used to get the data size returns 3 for an empty string, where valueIn[2]
+				is a ? mark. 
+				*/
+
+				if (valuelenIn > 2) {
+					if ((valueIn[0] == 0x0) && (valueIn[1] == 0x0)) {
+						for (DWORD x=2; x<(valuelenIn); x++) valueIn[x] = 0x0;
+					}
+				}
+
 				char strBuf[3];
 				string strValue = "";
 				for (DWORD x=0; x<(valuelenIn); x++) {
