@@ -1,8 +1,7 @@
 //
-// $Id: FileFinder.cpp 4164 2007-06-25 16:47:54Z bakerj $
 //
 //****************************************************************************************//
-// Copyright (c) 2002-2007, The MITRE Corporation
+// Copyright (c) 2002-2008, The MITRE Corporation
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without modification, are
@@ -42,6 +41,43 @@ FileFinder::~FileFinder() {
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  Private Members  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
+
+StringVector* FileFinder::ProcessPathBehaviors(StringVector* paths, BehaviorVector* behaviors) {
+	
+	// Process the behaviors to identify any additional paths.
+	// initialize these default values based on the defaults 
+	// set in the oval definitions schema
+	string recurseDirection = Behavior::GetBehaviorValue(behaviors, "recurse_direction");
+	if(recurseDirection.compare("") == 0) {
+		recurseDirection = "none";
+	}
+
+	string maxDepthStr = Behavior::GetBehaviorValue(behaviors, "max_depth");
+	int maxDepth = -1;
+	if(maxDepthStr.compare("") != 0) {
+		maxDepth = atoi(maxDepthStr.c_str());
+		if(maxDepth < -1) 
+			maxDepth = -1;
+	}
+
+	// only need to address recurseDirection up & down if maxDepth is not 0
+	StringVector* behaviorPaths = new StringVector();
+	if(recurseDirection.compare("up") == 0 && maxDepth != 0) {
+		StringVector::iterator path;
+		for(path = paths->begin(); path != paths->end(); path++) {
+			this->UpwardPathRecursion(behaviorPaths, (*path), maxDepth);
+		}
+
+	} else if(recurseDirection.compare("down") == 0 && maxDepth != 0) {
+		StringVector::iterator path;
+		for(path = paths->begin(); path != paths->end(); path++) {
+			this->DownwardPathRecursion(behaviorPaths, (*path), maxDepth);
+		}
+	}
+
+	return behaviorPaths;
+}
+
 void FileFinder::FindPaths(string regex, StringVector* paths, bool isRegex) {
 
 	StringVector* drives;
@@ -192,12 +228,25 @@ void FileFinder::GetPathsForPattern(string dirIn, string pattern, StringVector *
 		hFind = FindFirstFile(findDir.c_str(), &FindFileData);
 		if (hFind == INVALID_HANDLE_VALUE) {
 
-			string errorMessage = "";
-			errorMessage.append("Error: Unable to get a valid handle in GetPathsForPattern().\n\tDirectory: ");
-			errorMessage.append(dirIn);
-			errorMessage.append("\n\tPattern: ");
-			errorMessage.append(pattern);
-			throw FileFinderException(errorMessage);
+			DWORD errorNum = GetLastError();
+
+			if(errorNum == ERROR_FILE_NOT_FOUND || errorNum == ERROR_PATH_NOT_FOUND) {
+
+				// if the file is not found just return no need to report an error
+				return;
+
+			} else {
+				
+				// report other errors that might occure
+				string msg = WindowsCommon::GetErrorMessage(errorNum);
+				string errorMessage = "";
+				errorMessage.append("Error while seaching for matching file paths. " + msg);
+				errorMessage.append(" Directory: ");
+				errorMessage.append(dirIn);
+				errorMessage.append(" Pattern: ");
+				errorMessage.append(pattern);
+				throw FileFinderException(errorMessage);
+			}
 		}
 
 		//	Loop through each file in the directory.  
@@ -228,13 +277,15 @@ void FileFinder::GetPathsForPattern(string dirIn, string pattern, StringVector *
 
 		//	Close the handle to the file search object.
 		if(!FindClose(hFind)) {
-			string errorMessage = "";
-			errorMessage.append("Error: ");
-			errorMessage.append("Unable to close search handle while trying to search for matching paths. \n\tDirectory: ");
-			errorMessage.append(dirIn);
-			errorMessage.append("\n\tPattern: ");
-			errorMessage.append(pattern);
 
+			DWORD errorNum = GetLastError();
+			string msg = WindowsCommon::GetErrorMessage(errorNum);
+			string errorMessage = "";
+			errorMessage.append("Error: Unable to close search handle while trying to search for matching paths. " + msg);
+			errorMessage.append(" Directory: ");
+			errorMessage.append(dirIn);
+			errorMessage.append(" Pattern: ");
+			errorMessage.append(pattern);
 			throw FileFinderException(errorMessage);	
 		}
 
@@ -248,9 +299,9 @@ void FileFinder::GetPathsForPattern(string dirIn, string pattern, StringVector *
 
 		string errorMessage = "";
 		errorMessage.append("Error: ");
-		errorMessage.append("An unspecified error was encountered while trying to search for matching paths. \n\tDirectory: ");
+		errorMessage.append("An unspecified error was encountered while trying to search for matching paths. Directory: ");
 		errorMessage.append(dirIn);
-		errorMessage.append("\n\tPattern: ");
+		errorMessage.append(" Pattern: ");
 		errorMessage.append(pattern);
 		throw FileFinderException(errorMessage);
 	}
@@ -278,12 +329,25 @@ void FileFinder::GetFilesForPattern(string path, string pattern, StringVector* f
 		hFind = FindFirstFile(findDir.c_str(), &FindFileData);
 		if (hFind == INVALID_HANDLE_VALUE) {
 
-			string errorMessage = "";
-			errorMessage.append("Error: Unable to get a valid handle in GetFilesForPattern().\n\tDirectory: ");
-			errorMessage.append(path);
-			errorMessage.append("\n\tPattern: ");
-			errorMessage.append(pattern);
-			throw FileFinderException(errorMessage);
+			DWORD errorNum = GetLastError();
+
+			if(errorNum == ERROR_FILE_NOT_FOUND || errorNum == ERROR_PATH_NOT_FOUND) {
+
+				// if the file is not found just return no need to report an error
+				return;
+
+			} else {
+				
+				// report other errors that might occure
+				string msg = WindowsCommon::GetErrorMessage(errorNum);
+				string errorMessage = "";
+				errorMessage.append("Error while seaching for matching files. " + msg);
+				errorMessage.append(" Directory: ");
+				errorMessage.append(path);
+				errorMessage.append(" Pattern: ");
+				errorMessage.append(pattern);
+				throw FileFinderException(errorMessage);
+			}
 		}
 
 		//	Loop through each file in the directory.  
@@ -313,13 +377,16 @@ void FileFinder::GetFilesForPattern(string path, string pattern, StringVector* f
 
 		//	Close the handle to the file search object.
 		if(!FindClose(hFind)) {
+
+			DWORD errorNum = GetLastError();
+			string msg = WindowsCommon::GetErrorMessage(errorNum);
 			string errorMessage = "";
-			errorMessage.append("Error: ");
-			errorMessage.append("Unable to close search handle while trying to search for matching file paths. \n\tDirectory: ");
+			errorMessage.append("Error: Unable to close search handle while trying to search for matching files. " + msg);
+			errorMessage.append(" Directory: ");
 			errorMessage.append(path);
-			errorMessage.append("\n\tPattern: ");
+			errorMessage.append(" Pattern: ");
 			errorMessage.append(pattern);
-			throw FileFinderException(errorMessage);	
+			throw FileFinderException(errorMessage);
 		}
 
 	//	Just need to ensure that all exceptions have a nice message. 
