@@ -1,8 +1,7 @@
 //
-// $Id: WindowsCommon.cpp 4094 2007-06-22 13:12:33Z bakerj $
 //
 //****************************************************************************************//
-// Copyright (c) 2002-2007, The MITRE Corporation
+// Copyright (c) 2002-2008, The MITRE Corporation
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without modification, are
@@ -304,7 +303,7 @@ bool WindowsCommon::GetTextualSid(PSID pSid, LPTSTR* TextualSid) {
     return true;
 }
 
- bool WindowsCommon::ExpandGroup(string groupName, StringVector* members) {
+bool WindowsCommon::ExpandGroup(string groupName, StringVector* members) {
 
 	// Need to determine if a local or global group.
 	bool groupExists = false;
@@ -378,7 +377,7 @@ bool WindowsCommon::GetLocalGroupMembers(string groupName, StringVector* members
 			throw Exception("Unable to expand local group: " + groupName + ". The computer name is invalid.");
 		} else if(res == ERROR_MORE_DATA) {
 			// throw this error
-			throw Exception("Unable to expand local group: " + groupName + ". More entries are available. Specify a large enough buffer to receive all entries. This error message should never occure since the api call is made with MAX_PREFERRED_LENGTH for the size of teh buffer.");
+			throw Exception("Unable to expand local group: " + groupName + ". More entries are available. Specify a large enough buffer to receive all entries. This error message should never occure since the api call is made with MAX_PREFERRED_LENGTH for the size of the buffer.");
 		} else if(res == ERROR_NO_SUCH_ALIAS || res == NERR_GroupNotFound) {
 			// ignore this error
 			//cout << "The specified local group " + groupName + " does not exist." << endl;
@@ -449,7 +448,7 @@ bool WindowsCommon::GetGlobalGroupMembers(string groupName, StringVector* member
 			throw Exception("Unable to expand global group: " + groupName + ". The computer name is invalid.");
 		} else if(res == ERROR_MORE_DATA) {
 			// throw this error
-			throw Exception("Unable to expand global group: " + groupName + ". More entries are available. Specify a large enough buffer to receive all entries. This error message should never occure since the api call is made with MAX_PREFERRED_LENGTH for the size of teh buffer.");
+			throw Exception("Unable to expand global group: " + groupName + ". More entries are available. Specify a large enough buffer to receive all entries. This error message should never occure since the api call is made with MAX_PREFERRED_LENGTH for the size of the buffer.");
 		} else if(res == NERR_GroupNotFound) {
 			groupExists = false;
 			// no action here
@@ -1047,4 +1046,20 @@ string WindowsCommon::LookUpLocalSystemName() {
 	}
 
 	return systemName;
-}
\ No newline at end of file
+}
+
+string WindowsCommon::ToString(FILETIME fTime) {
+
+	return Common::ToString(fTime.dwLowDateTime) + Common::ToString(fTime.dwHighDateTime);
+}
+
+string WindowsCommon::ToString(DWORD dw) {
+
+	char dwordBuf[12];
+	ZeroMemory(dwordBuf, sizeof(dwordBuf));
+	_snprintf(dwordBuf, sizeof(dwordBuf)-1, "%d", dw);
+	dwordBuf[sizeof(dwordBuf)-1] = '\0';
+
+	string dwStr = dwordBuf;
+	return dwStr;
+}
